#include <math.h>
#include <stdio.h>
#include <string.h>

const double	PI = 3.1415926535897932384626433832795;

void GetDataLUT(double wavelength, float wavelengthLUT,
		int numRadiiGridBins, float *pRadiiGridBins,
		int numScatteringAngles,
		int recordIndex,
		double *dV_dlnr,
		double *pCoef1, double *pCoef2, double *pCoef3,
		float **ppCext, double *pExtCoef, float **ppCsca, double *pScaCoef,
		float ***pppC11, double *pP11, float ***pppC12, double *pP12,
		float ***pppC33, double *pP33, float ***pppC34, double *pP34)
{
	int	i, k, l, m;
	double	kernelExt, kernelSca, radius, wavelengthRatio;
	double	*pKernel11, *pKernel12, *pKernel33, *pKernel34;

	pKernel11 = new double[numScatteringAngles];
	pKernel12 = new double[numScatteringAngles];
	pKernel33 = new double[numScatteringAngles];
	pKernel34 = new double[numScatteringAngles];

	wavelengthRatio = wavelengthLUT/wavelength;

	memset(pP11, 0, numScatteringAngles*sizeof(double));
	memset(pP12, 0, numScatteringAngles*sizeof(double));
	memset(pP33, 0, numScatteringAngles*sizeof(double));
	memset(pP34, 0, numScatteringAngles*sizeof(double));
	for (pExtCoef[0] = pScaCoef[0] = i = k = 0; i < numRadiiGridBins; i++) {
		radius = wavelengthRatio*pRadiiGridBins[i];
		for (l = k; l < numRadiiGridBins - 1; l++)
			if (radius >= pRadiiGridBins[l] && radius <= pRadiiGridBins[l + 1]) {
				if (l) {
					kernelExt = ppCext[recordIndex][l - 1]*pCoef1[i] + ppCext[recordIndex][l]*pCoef2[i] + ppCext[recordIndex][l + 1]*pCoef3[i];
					kernelSca = ppCsca[recordIndex][l - 1]*pCoef1[i] + ppCsca[recordIndex][l]*pCoef2[i] + ppCsca[recordIndex][l + 1]*pCoef3[i];
					for (m = 0; m < numScatteringAngles; m++) {
						pKernel11[m] = pppC11[recordIndex][l - 1][m]*pCoef1[i] + pppC11[recordIndex][l][m]*pCoef2[i] + pppC11[recordIndex][l + 1][m]*pCoef3[i];
						pKernel12[m] = pppC12[recordIndex][l - 1][m]*pCoef1[i] + pppC12[recordIndex][l][m]*pCoef2[i] + pppC12[recordIndex][l + 1][m]*pCoef3[i];
						pKernel33[m] = pppC33[recordIndex][l - 1][m]*pCoef1[i] + pppC33[recordIndex][l][m]*pCoef2[i] + pppC33[recordIndex][l + 1][m]*pCoef3[i];
						pKernel34[m] = pppC34[recordIndex][l - 1][m]*pCoef1[i] + pppC34[recordIndex][l][m]*pCoef2[i] + pppC34[recordIndex][l + 1][m]*pCoef3[i];
					}
				}
				else {
					kernelExt = ppCext[recordIndex][l]*pCoef1[i] + ppCext[recordIndex][l + 1]*pCoef2[i] + ppCext[recordIndex][l + 2]*pCoef3[i];
					kernelSca = ppCsca[recordIndex][l]*pCoef1[i] + ppCsca[recordIndex][l + 1]*pCoef2[i] + ppCsca[recordIndex][l + 2]*pCoef3[i];
					for (m = 0; m < numScatteringAngles; m++) {
						pKernel11[m] = pppC11[recordIndex][l][m]*pCoef1[i] + pppC11[recordIndex][l + 1][m]*pCoef2[i] + pppC11[recordIndex][l + 2][m]*pCoef3[i];
						pKernel12[m] = pppC12[recordIndex][l][m]*pCoef1[i] + pppC12[recordIndex][l + 1][m]*pCoef2[i] + pppC12[recordIndex][l + 2][m]*pCoef3[i];
						pKernel33[m] = pppC33[recordIndex][l][m]*pCoef1[i] + pppC33[recordIndex][l + 1][m]*pCoef2[i] + pppC33[recordIndex][l + 2][m]*pCoef3[i];
						pKernel34[m] = pppC34[recordIndex][l][m]*pCoef1[i] + pppC34[recordIndex][l + 1][m]*pCoef2[i] + pppC34[recordIndex][l + 2][m]*pCoef3[i];
					}
				}
				pExtCoef[0] += kernelExt*dV_dlnr[i];
				pScaCoef[0] += kernelSca*dV_dlnr[i];
				for (m = 0; m < numScatteringAngles; m++) {
					pP11[m] += pKernel11[m]*dV_dlnr[i];
					pP12[m] += pKernel12[m]*dV_dlnr[i];
					pP33[m] += pKernel33[m]*dV_dlnr[i];
					pP34[m] += pKernel34[m]*dV_dlnr[i];
				}
				k = l;	break;
			}
	}
	pExtCoef[0] *= wavelengthRatio;
	pScaCoef[0] *= wavelengthRatio;
	for (m = 0; m < numScatteringAngles; m++) {
		pP11[m] *= wavelengthRatio;
		pP12[m] *= wavelengthRatio;
		pP33[m] *= wavelengthRatio;
		pP34[m] *= wavelengthRatio;
	}
	delete[] pKernel11;	delete[] pKernel12;	delete[] pKernel33;	delete[] pKernel34;
}

int main()
{
	char	charBuf[1024];
	double	buf, buf11[123], buf12[123], buf33[123], buf34[123], bufExt, bufSca, coefImag[3], coefReal[3],
		coefRealImag, countMedianRadius[2], currentAngle, final4, gFinal, imagPart[2], lnSigma[2], nFinal,
		numberConcentration[2], realPart[2], reffFinal, sFinal, veffFinal, vFinal, x1, x2, x3, y1, y2, y3;
	double	absCoefFinal, asymmetryFinal, bscCoefFinal, extCoef[2], extCoefFinal, scaCoef[2], scaCoefFinal, ssaFinal,
		p11[2][123], p11Final[123], p12[2][123], p12Final[123], p33[2][123], p33Final[123], p34[2][123], p34Final[123];
	double	*dV_dlnr, *pCoef1, *pCoef2, *pCoef3, radius, wavelength, wavelengthRatio;
	FILE	*pFile;
	float	**ppCext, **ppCsca, ***pppC11, ***pppC12, ***pppC33, ***pppC34, *pRadiiGridBinsLUT, *pScatteringAnglesLUT,
		*pRealPartRecord, *pRealPartsLUT, *pImagPartRecord, *pImagPartsLUT, wavelengthLUT;
	int	i, iGB, iI, iM, iR, imagPartIndex[3], j, k, l, m, numberOfModes, numCRI, numImagPartsLUT, numRadiiGridBinsLUT,
		numRealPartsLUT, numIntegrationAngles = 10000, numScatteringAnglesLUT, realPartIndex[3];

	// Change the desired wavelength (mum)
	wavelength = 0.55;

	// Bimodal   aerosol if numberOfModes = 2
	// Monomodal aerosol if numberOfModes = 1
	numberOfModes = 2;

	// Complex refractive index
	realPart[0] = 1.55;
	realPart[1] = 1.52;
	imagPart[0] = 0.04;
	imagPart[1] = 0.0019;

	// Lognormal PSD
	countMedianRadius[0] = 0.12;
	countMedianRadius[1] = 0.5;
	lnSigma[0] = 0.37;
	lnSigma[1] = 0.75;
	numberConcentration[0] = 10;
	numberConcentration[1] = 5;

	pFile = fopen("AerosolLUT_1000_100_0.355_650bins_2325CRI_ln2rKr_Twomey.dat", "rb+");
	fread(&wavelengthLUT, 1, sizeof(wavelengthLUT), pFile);
	fread(&numRadiiGridBinsLUT, 1, sizeof(numRadiiGridBinsLUT), pFile);
	pRadiiGridBinsLUT = new float[numRadiiGridBinsLUT];
	fread(pRadiiGridBinsLUT, 1, numRadiiGridBinsLUT*sizeof(float), pFile);
	fread(&numScatteringAnglesLUT, 1, sizeof(numScatteringAnglesLUT), pFile);
	pScatteringAnglesLUT = new float[numScatteringAnglesLUT];
	fread(pScatteringAnglesLUT, 1, numScatteringAnglesLUT*sizeof(float), pFile);
	fread(&numRealPartsLUT, 1, sizeof(numRealPartsLUT), pFile);
	pRealPartsLUT = new float[numRealPartsLUT];
	fread(pRealPartsLUT, 1, numRealPartsLUT*sizeof(float), pFile);
	fread(&numImagPartsLUT, 1, sizeof(numImagPartsLUT), pFile);
	pImagPartsLUT = new float[numImagPartsLUT];
	fread(pImagPartsLUT, 1, numImagPartsLUT*sizeof(float), pFile);
	numCRI = numRealPartsLUT*numImagPartsLUT;
	pRealPartRecord = new float[numCRI];
	pImagPartRecord = new float[numCRI];
	ppCext = new float*[numCRI];
	ppCsca = new float*[numCRI];
	pppC11 = new float**[numCRI];
	pppC12 = new float**[numCRI];
	pppC33 = new float**[numCRI];
	pppC34 = new float**[numCRI];
	for (i = 0; i < numCRI; i++) {
		ppCext[i] = new float[numRadiiGridBinsLUT];
		ppCsca[i] = new float[numRadiiGridBinsLUT];
		pppC11[i] = new float*[numRadiiGridBinsLUT];
		pppC12[i] = new float*[numRadiiGridBinsLUT];
		pppC33[i] = new float*[numRadiiGridBinsLUT];
		pppC34[i] = new float*[numRadiiGridBinsLUT];
		for (j = 0; j < numRadiiGridBinsLUT; j++) {
			pppC11[i][j] = new float[numScatteringAnglesLUT];
			pppC12[i][j] = new float[numScatteringAnglesLUT];
			pppC33[i][j] = new float[numScatteringAnglesLUT];
			pppC34[i][j] = new float[numScatteringAnglesLUT];
		}
	}
	for (i = 0; i < numCRI; i++) {
		fread(&pRealPartRecord[i], 1, sizeof(float), pFile);
		fread(&pImagPartRecord[i], 1, sizeof(float), pFile);
		fread(ppCext[i], 1, numRadiiGridBinsLUT*sizeof(float), pFile);
		fread(ppCsca[i], 1, numRadiiGridBinsLUT*sizeof(float), pFile);
		for (iGB = 0; iGB < numRadiiGridBinsLUT; iGB++)	fread(pppC11[i][iGB], 1, numScatteringAnglesLUT*sizeof(float), pFile);
		for (iGB = 0; iGB < numRadiiGridBinsLUT; iGB++)	fread(pppC12[i][iGB], 1, numScatteringAnglesLUT*sizeof(float), pFile);
		for (iGB = 0; iGB < numRadiiGridBinsLUT; iGB++)	fread(pppC33[i][iGB], 1, numScatteringAnglesLUT*sizeof(float), pFile);
		for (iGB = 0; iGB < numRadiiGridBinsLUT; iGB++)	fread(pppC34[i][iGB], 1, numScatteringAnglesLUT*sizeof(float), pFile);
	}
	fclose(pFile);

	wavelengthRatio = wavelengthLUT/wavelength;
	pCoef1 = new double[numRadiiGridBinsLUT];
	pCoef2 = new double[numRadiiGridBinsLUT];
	pCoef3 = new double[numRadiiGridBinsLUT];
	memset(pCoef1, 0, numRadiiGridBinsLUT*sizeof(double));
	memset(pCoef2, 0, numRadiiGridBinsLUT*sizeof(double));
	memset(pCoef3, 0, numRadiiGridBinsLUT*sizeof(double));
	for (i = k = 0; i < numRadiiGridBinsLUT; i++) {
		radius = wavelengthRatio*pRadiiGridBinsLUT[i];
		for (l = k; l < numRadiiGridBinsLUT - 1; l++)
			if (radius >= pRadiiGridBinsLUT[l] && radius <= pRadiiGridBinsLUT[l + 1]) {
				if (l)	{x1 = pRadiiGridBinsLUT[l - 1];	x2 = pRadiiGridBinsLUT[l];	x3 = pRadiiGridBinsLUT[l + 1];}
				else	{x1 = pRadiiGridBinsLUT[l];	x2 = pRadiiGridBinsLUT[l + 1];	x3 = pRadiiGridBinsLUT[l + 2];}
				pCoef1[i] = (radius - x2)*(radius - x3)/(x1 - x2)/(x1 - x3);
				pCoef2[i] = (radius - x1)*(radius - x3)/(x2 - x1)/(x2 - x3);
				pCoef3[i] = (radius - x1)*(radius - x2)/(x3 - x1)/(x3 - x2);
				k = l;	break;
			}
	}

	dV_dlnr = new double[numRadiiGridBinsLUT];

	memset(p11, 0, sizeof(p11));	memset(p11Final, 0, sizeof(p11Final));
	memset(p12, 0, sizeof(p12));	memset(p12Final, 0, sizeof(p12Final));
	memset(p33, 0, sizeof(p33));	memset(p33Final, 0, sizeof(p33Final));
	memset(p34, 0, sizeof(p34));	memset(p34Final, 0, sizeof(p34Final));
	extCoefFinal = final4 = gFinal = nFinal = reffFinal = scaCoefFinal = sFinal = veffFinal = vFinal = 0;
	for (iM = 0; iM < numberOfModes; iM++) {
		for (iGB = 0; iGB < numRadiiGridBinsLUT; iGB++)
			dV_dlnr[iGB] = numberConcentration[iM]*4.*PI/3.*pRadiiGridBinsLUT[iGB]*pRadiiGridBinsLUT[iGB]*pRadiiGridBinsLUT[iGB]/sqrt(2.*PI)/lnSigma[iM]*exp( -log(pRadiiGridBinsLUT[iGB]/countMedianRadius[iM])*log(pRadiiGridBinsLUT[iGB]/countMedianRadius[iM])/2./lnSigma[iM]/lnSigma[iM] );

		for (iR = 0; iR < numRealPartsLUT - 1; iR++)
			if (realPart[iM] >= pRealPartsLUT[iR] && realPart[iM] <= pRealPartsLUT[iR + 1])	break;
		if (realPart[iM] < pRealPartsLUT[0])			iR = 0;
		if (realPart[iM] > pRealPartsLUT[numRealPartsLUT - 1])	iR = numRealPartsLUT - 2;
		if (iR) {
			x1 = pRealPartsLUT[iR - 1];	realPartIndex[0] = iR - 1;
			x2 = pRealPartsLUT[iR];		realPartIndex[1] = iR;
			x3 = pRealPartsLUT[iR + 1];	realPartIndex[2] = iR + 1;
		}
		else {
			x1 = pRealPartsLUT[iR];		realPartIndex[0] = iR;
			x2 = pRealPartsLUT[iR + 1];	realPartIndex[1] = iR + 1;
			x3 = pRealPartsLUT[iR + 2];	realPartIndex[2] = iR + 2;
		}
		coefReal[0] = (realPart[iM] - x2)*(realPart[iM] - x3)/(x1 - x2)/(x1 - x3);
		coefReal[1] = (realPart[iM] - x1)*(realPart[iM] - x3)/(x2 - x1)/(x2 - x3);
		coefReal[2] = (realPart[iM] - x1)*(realPart[iM] - x2)/(x3 - x1)/(x3 - x2);

		for (iI = 0; iI < numImagPartsLUT - 1; iI++)
			if (imagPart[iM] >= pImagPartsLUT[iI] && imagPart[iM] <= pImagPartsLUT[iI + 1])	break;
		if (imagPart[iM] < pImagPartsLUT[0])			iI = 0;
		if (imagPart[iM] > pImagPartsLUT[numImagPartsLUT - 1])	iI = numImagPartsLUT - 2;
		if (iI) {
			x1 = pImagPartsLUT[iI - 1];	imagPartIndex[0] = iI - 1;
			x2 = pImagPartsLUT[iI];		imagPartIndex[1] = iI;
			x3 = pImagPartsLUT[iI + 1];	imagPartIndex[2] = iI + 1;
		}
		else {
			x1 = pImagPartsLUT[iI];		imagPartIndex[0] = iI;
			x2 = pImagPartsLUT[iI + 1];	imagPartIndex[1] = iI + 1;
			x3 = pImagPartsLUT[iI + 2];	imagPartIndex[2] = iI + 2;
		}
		coefImag[0] = (imagPart[iM] - x2)*(imagPart[iM] - x3)/(x1 - x2)/(x1 - x3);
		coefImag[1] = (imagPart[iM] - x1)*(imagPart[iM] - x3)/(x2 - x1)/(x2 - x3);
		coefImag[2] = (imagPart[iM] - x1)*(imagPart[iM] - x2)/(x3 - x1)/(x3 - x2);

		for (extCoef[iM] = scaCoef[iM] = j = 0; j < 3; j++)
			for (k = 0; k < 3; k++) {
				GetDataLUT(wavelength, wavelengthLUT,
						numRadiiGridBinsLUT, pRadiiGridBinsLUT,
						numScatteringAnglesLUT,
						numImagPartsLUT*realPartIndex[j] + imagPartIndex[k],
						dV_dlnr,
						pCoef1, pCoef2, pCoef3,
						ppCext, &bufExt, ppCsca, &bufSca,
						pppC11, &buf11[0], pppC12, &buf12[0],
						pppC33, &buf33[0], pppC34, &buf34[0]);
				coefRealImag = coefReal[j]*coefImag[k];
				extCoef[iM] += bufExt*coefRealImag;
				scaCoef[iM] += bufSca*coefRealImag;
				for (m = 0; m < numScatteringAnglesLUT; m++) {
					p11[iM][m] += buf11[m]*coefRealImag;
					p12[iM][m] += buf12[m]*coefRealImag;
					p33[iM][m] += buf33[m]*coefRealImag;
					p34[iM][m] += buf34[m]*coefRealImag;
				}
			}
		extCoefFinal += extCoef[iM];
		scaCoefFinal += scaCoef[iM];
		for (m = 0; m < numScatteringAnglesLUT; m++) {
			p11Final[m] += p11[iM][m];
			p12Final[m] += p12[iM][m];
			p33Final[m] += p33[iM][m];
			p34Final[m] += p34[iM][m];
		}
		gFinal += numberConcentration[iM]*PI*countMedianRadius[iM]*countMedianRadius[iM]*exp(2*lnSigma[iM]*lnSigma[iM]);
		nFinal += numberConcentration[iM];
		sFinal += numberConcentration[iM]*countMedianRadius[iM]*countMedianRadius[iM]*exp(2*lnSigma[iM]*lnSigma[iM]);
		vFinal += numberConcentration[iM]*countMedianRadius[iM]*countMedianRadius[iM]*countMedianRadius[iM]*exp(4.5*lnSigma[iM]*lnSigma[iM]);
		final4 += numberConcentration[iM]*countMedianRadius[iM]*countMedianRadius[iM]*countMedianRadius[iM]*countMedianRadius[iM]*exp(8*lnSigma[iM]*lnSigma[iM]);
	}
	if (nFinal > 0)					gFinal /= nFinal;
	if (fabs(sFinal) > 0)				reffFinal = vFinal/sFinal;
	if (fabs(sFinal) > 0 && fabs(reffFinal) > 0)	veffFinal = 1 + (final4 - 2*reffFinal*vFinal)/sFinal/reffFinal/reffFinal;
	absCoefFinal = fabs(extCoefFinal - scaCoefFinal);
	ssaFinal = scaCoefFinal/extCoefFinal;
	bscCoefFinal = p11Final[numScatteringAnglesLUT - 1]/4./PI;
	for (m = 0; m < numScatteringAnglesLUT; m++) {
		p12Final[m] /= p11Final[m];
		p33Final[m] /= p11Final[m];
		p34Final[m] /= p11Final[m];
		p11Final[m] /= scaCoefFinal;
	}
	for (asymmetryFinal = j = 0; j < numIntegrationAngles; j++) {
		currentAngle = j*180./double(numIntegrationAngles - 1);
		for (buf = m = 0; m < numScatteringAnglesLUT - 1; m++) {
			if (currentAngle >= pScatteringAnglesLUT[m] && currentAngle <= pScatteringAnglesLUT[m + 1]) {
				if (m < 0.5*numScatteringAnglesLUT) {
					x1 = pScatteringAnglesLUT[m];		y1 = log(p11Final[m]);
					x2 = pScatteringAnglesLUT[m + 1];	y2 = log(p11Final[m + 1]);
					x3 = pScatteringAnglesLUT[m + 2];	y3 = log(p11Final[m + 2]);
				}
				else {
					x1 = pScatteringAnglesLUT[m - 1];	y1 = log(p11Final[m - 1]);
					x2 = pScatteringAnglesLUT[m];		y2 = log(p11Final[m]);
					x3 = pScatteringAnglesLUT[m + 1];	y3 = log(p11Final[m + 1]);
				}
				buf = y1*(currentAngle - x2)*(currentAngle - x3)/(x1 - x2)/(x1 - x3) + y2*(currentAngle - x1)*(currentAngle - x3)/(x2 - x1)/(x2 - x3) + y3*(currentAngle - x1)*(currentAngle - x2)/(x3 - x1)/(x3 - x2);
				buf = exp(buf)*sin(currentAngle*PI/180.)*cos(currentAngle*PI/180.);
				break;
			}
		}
		//Numerical integration using Simpson's rule
		if (!j || j == numIntegrationAngles - 1)	asymmetryFinal += 0.5*buf;
		else {
			if (j%2)	asymmetryFinal += 2*buf;
			else		asymmetryFinal += buf;
		}
	}
	asymmetryFinal *= PI/double(numIntegrationAngles - 1)/3.;

	sprintf(charBuf, "Output_%14.12f.txt", wavelength);
	pFile = fopen(charBuf, "wb");
	sprintf(charBuf, "Wavelength = %14.12f mum\n\n", wavelength);
	fwrite(&charBuf[0], 1, strlen(charBuf), pFile);
	for (iM = 0; iM < numberOfModes; iM++) {
		sprintf(charBuf, "Aerosol mode %d\n", iM + 1);
		fwrite(&charBuf[0], 1, strlen(charBuf), pFile);
		sprintf(charBuf, "  Complex refractive index:\n");
		fwrite(&charBuf[0], 1, strlen(charBuf), pFile);
		sprintf(charBuf, "    real part = %14.12f\n", realPart[iM]);
		fwrite(&charBuf[0], 1, strlen(charBuf), pFile);
		sprintf(charBuf, "    imaginary part = %14.12f\n", imagPart[iM]);
		fwrite(&charBuf[0], 1, strlen(charBuf), pFile);
		sprintf(charBuf, "  Logarithmic-normal particle size distribution:\n");
		fwrite(&charBuf[0], 1, strlen(charBuf), pFile);
		sprintf(charBuf, "    total number concentration =%19.12e 1/cm^3\n", numberConcentration[iM]);
		fwrite(&charBuf[0], 1, strlen(charBuf), pFile);
		sprintf(charBuf, "    count median radius = %14.12f mum\n", countMedianRadius[iM]);
		fwrite(&charBuf[0], 1, strlen(charBuf), pFile);
		sprintf(charBuf, "    geometric standard deviation = %14.12f\n", exp(lnSigma[iM]));
		fwrite(&charBuf[0], 1, strlen(charBuf), pFile);
		sprintf(charBuf, "    <g> =%19.12e mum^2\n", PI*countMedianRadius[iM]*countMedianRadius[iM]*exp(2*lnSigma[iM]*lnSigma[iM]));
		fwrite(&charBuf[0], 1, strlen(charBuf), pFile);
		sprintf(charBuf, "    effective radius =%19.12e mum\n", countMedianRadius[iM]*exp(2.5*lnSigma[iM]*lnSigma[iM]));
		fwrite(&charBuf[0], 1, strlen(charBuf), pFile);
		sprintf(charBuf, "    effective variance =%19.12e\n", exp(lnSigma[iM]*lnSigma[iM]) - 1);
		fwrite(&charBuf[0], 1, strlen(charBuf), pFile);
	}
	sprintf(charBuf, "\nOptical coefficients:\n");
	fwrite(&charBuf[0], 1, strlen(charBuf), pFile);
	sprintf(charBuf, "  absorption     =%19.12e 1/Mm\n", absCoefFinal);
	fwrite(&charBuf[0], 1, strlen(charBuf), pFile);
	sprintf(charBuf, "  extinction     =%19.12e 1/Mm\n", extCoefFinal);
	fwrite(&charBuf[0], 1, strlen(charBuf), pFile);
	sprintf(charBuf, "  scattering     =%19.12e 1/Mm\n", scaCoefFinal);
	fwrite(&charBuf[0], 1, strlen(charBuf), pFile);
	sprintf(charBuf, "  backscattering =%19.12e 1/(Mm sr)\n", bscCoefFinal);
	fwrite(&charBuf[0], 1, strlen(charBuf), pFile);
	sprintf(charBuf, "Single-scattering albedo = %14.12f\n", ssaFinal);
	fwrite(&charBuf[0], 1, strlen(charBuf), pFile);
	sprintf(charBuf, "Asymmetry parameter =%19.12e\n", asymmetryFinal);
	fwrite(&charBuf[0], 1, strlen(charBuf), pFile);
	sprintf(charBuf, "<G> =%19.12e mum^2\n", gFinal);
	fwrite(&charBuf[0], 1, strlen(charBuf), pFile);
	sprintf(charBuf, "Effective radius =%19.12e mum\n", reffFinal);
	fwrite(&charBuf[0], 1, strlen(charBuf), pFile);
	sprintf(charBuf, "Effective variance =%19.12e\n\n", veffFinal);
	fwrite(&charBuf[0], 1, strlen(charBuf), pFile);
	sprintf(charBuf, "            ******* Normalized scattering matrix *******\n");
	fwrite(&charBuf[0], 1, strlen(charBuf), pFile);
	sprintf(charBuf, "Angle  P11           P12/P11       P33/P11       P34/P11\n");
	fwrite(&charBuf[0], 1, strlen(charBuf), pFile);
	for (m = 0; m < numScatteringAnglesLUT; m++) {
		sprintf(charBuf, "%5.1f %13.6e %13.6e %13.6e %13.6e\n", pScatteringAnglesLUT[m], p11Final[m], p12Final[m], p33Final[m], p34Final[m]);
		fwrite(&charBuf[0], 1, strlen(charBuf), pFile);
	}
	fclose(pFile);

	delete[] pRadiiGridBinsLUT;
	delete[] pScatteringAnglesLUT;
	delete[] pRealPartsLUT;
	delete[] pImagPartsLUT;
	delete[] pRealPartRecord;
	delete[] pImagPartRecord;
	for (i = 0; i < numCRI; i++) {
		delete[] ppCext[i];
		delete[] ppCsca[i];
		for (j = 0; j < numRadiiGridBinsLUT; j++) {
			delete[] pppC11[i][j];	delete[] pppC12[i][j];	delete[] pppC33[i][j];	delete[] pppC34[i][j];
		}
		delete[] pppC11[i];	delete[] pppC12[i];	delete[] pppC33[i];	delete[] pppC34[i];
	}
	delete[] ppCext;	delete[] ppCsca;
	delete[] pppC11;	delete[] pppC12;	delete[] pppC33;	delete[] pppC34;
	delete[] pCoef1;	delete[] pCoef2;	delete[] pCoef3;
	delete[] dV_dlnr;

	return 0;
}