! Subroutine estimates all the necessary parameters using look-up table
subroutine sir_sca_driver(wavelength, iW, realPart, imagPart, &
                          ndistr, numberOfModes, psdParam1, psdParam2, psdParam3)

  use sir_sca_store
  implicit none

  integer, intent(in) :: ndistr, numberOfModes, iW
  real*8,  intent(in) :: imagPart(4), psdParam1(4), psdParam2(4), psdParam3(4), realPart(4), wavelength

  integer iA, iGB, iI, iL, imagPartIndex(3), iP, iR, istat, j, k, numberIntegrationAngles, realPartIndex(3), recordIndex
  logical lCRIFound
  real*8  countMedianRadius(4), lnSigma(4), numberConcentration(4), &
          buf, curAngle, coefReal(3), coefImag(3), coefRI, extBuf, scaBuf, x1, x2, x3, y1, y2, y3, &
          p11Buf(500), p12Buf(500), p22Buf(500), p33Buf(500), p34Buf(500), p44Buf(500), &
          bufG, bufN, bufReff, bufS, bufV, bufVeff, buf4, dVdln, final4, hln, nFinal, sFinal, vFinal

  if (ndistr.eq.2) then ! Lognormal PSD
    do iP = 1, 4
      countMedianRadius(iP) = psdParam1(iP)
      lnSigma(iP) = psdParam2(iP)
      numberConcentration(iP) = psdParam3(iP)
    enddo
  endif ! if (ndistr.eq.2)

  lut_absCoefFinal(iW) = 0d0
  lut_asymmetryParamFinal(iW) = 0d0
  lut_extCoefFinal(iW) = 0d0
  lut_scaCoefFinal(iW) = 0d0
  lut_ssaFinal(iW) = 0d0
  lut_GFinal = 0d0
  lut_reffFinal = 0d0
  lut_veffFinal = 0d0
  do iA = 1, lut_numScatteringAngles
    lut_p11Final(iA, iW) = 0d0
    lut_p12Final(iA, iW) = 0d0
    lut_p22Final(iA, iW) = 0d0
    lut_p33Final(iA, iW) = 0d0
    lut_p34Final(iA, iW) = 0d0
    lut_p44Final(iA, iW) = 0d0
  enddo
  do iP = 1, lut_numPSD
    lut_G(iP) = 0d0
    lut_reff(iP) = 0d0
    lut_veff(iP) = 0d0
    do iGB = 1, lut_numRadiiGridBins
      lut_dV_dlnr(iGB, iP) = 0d0
    enddo
    do iL = 1, lut_numLUT
      lut_absCoef(iP, iW, iL) = 0d0
      lut_extCoef(iP, iW, iL) = 0d0
      lut_scaCoef(iP, iW, iL) = 0d0
      lut_ssa(iP, iW, iL) = 0d0
      do iA = 1, lut_numScatteringAngles
        lut_p11(iA, iP, iW, iL) = 0d0
        lut_p12(iA, iP, iW, iL) = 0d0
        lut_p22(iA, iP, iW, iL) = 0d0
        lut_p33(iA, iP, iW, iL) = 0d0
        lut_p34(iA, iP, iW, iL) = 0d0
        lut_p44(iA, iP, iW, iL) = 0d0
      enddo
    enddo
  enddo

  nFinal = 0d0
  sFinal = 0d0
  vFinal = 0d0
  final4 = 0d0
  do iP = 1, numberOfModes
    if (ndistr.eq.2) then ! Lognormal PSD
      lut_G(iP) = lut_pi*countMedianRadius(iP)*countMedianRadius(iP)*exp(2d0*lnSigma(iP)*lnSigma(iP))
      lut_GFinal = lut_GFinal + numberConcentration(iP)*lut_G(iP)
      nFinal = nFinal + numberConcentration(iP)
      lut_reff(iP) = countMedianRadius(iP)*exp(2.5d0*lnSigma(iP)*lnSigma(iP))
      lut_veff(iP) = exp(lnSigma(iP)*lnSigma(iP)) - 1
      sFinal = sFinal + numberConcentration(iP)*countMedianRadius(iP)*countMedianRadius(iP)*exp(2d0*lnSigma(iP)*lnSigma(iP))
      vFinal = vFinal + numberConcentration(iP)* &
               countMedianRadius(iP)*countMedianRadius(iP)*countMedianRadius(iP)*exp(4.5d0*lnSigma(iP)*lnSigma(iP))
      final4 = final4 + numberConcentration(iP)*countMedianRadius(iP)* &
               countMedianRadius(iP)*countMedianRadius(iP)*countMedianRadius(iP)*exp(8d0*lnSigma(iP)*lnSigma(iP))
      do iGB = 1, lut_numRadiiGridBins
        lut_dV_dlnr(iGB, iP) = 4d0*lut_pi/3d0*lut_radiiGridBins(iGB)*lut_radiiGridBins(iGB)*lut_radiiGridBins(iGB)* &
                              numberConcentration(iP)/sqrt(2d0*lut_pi)/lnSigma(iP)* &
                              exp(-(log(lut_radiiGridBins(iGB)) - log(countMedianRadius(iP)))* &
                              (log(lut_radiiGridBins(iGB)) - log(countMedianRadius(iP)))/2d0/lnSigma(iP)/lnSigma(iP))
      enddo

      if (lut_useNumerical) then
        hln = (log(lut_radiiGridBins(lut_numRadiiGridBins)) - log(lut_radiiGridBins(1)))/(lut_numRadiiGridBins - 1d0)
        bufN = 0d0
        bufS = 0d0
        bufV = 0d0
        buf4 = 0d0
        do iGB = 1, lut_numRadiiGridBins
          bufN = bufN + 3d0*lut_dV_dlnr(iGB, iP)/4d0/lut_pi/ &
                        lut_radiiGridBins(iGB)/lut_radiiGridBins(iGB)/lut_radiiGridBins(iGB)
          bufS = bufS + 3d0*lut_dV_dlnr(iGB, iP)/lut_radiiGridBins(iGB)
          bufV = bufV + lut_dV_dlnr(iGB, iP)
          buf4 = buf4 + 3d0*lut_dV_dlnr(iGB, iP)/4d0/lut_pi*lut_radiiGridBins(iGB)
        enddo
        bufN = bufN*hln
        bufS = bufS*hln
        bufV = bufV*hln
        buf4 = buf4*hln
        bufReff = 0d0
        if (abs(bufS).gt.0d0)  bufReff = 3d0*bufV/bufS
        bufG = 0d0
        if (abs(bufN).gt.0d0)  bufG = bufS/bufN/4d0
        bufVeff = 0d0
        if (abs(bufS).gt.0d0.and.abs(bufReff).gt.0d0)  bufVeff = 4d0*lut_pi*buf4/bufS/bufReff/bufReff - 1d0
        lut_G(iP) = bufG
        lut_reff(iP) = bufReff
        lut_veff(iP) = bufVeff
      endif ! if (lut_useNumerical) then
    endif ! if (ndistr.eq.2)

    lCRIFound = .false.
    do iR = 1, lut_numRealParts - 1
      if (realPart(iP).ge.lut_realPartsLUT(iR).and.realPart(iP).le.lut_realPartsLUT(iR + 1)) then
        lCRIFound = .true.
        exit
      endif
    enddo
    if (.not.lCRIFound) then
      if (realPart(iP).lt.lut_realPartsLUT(1)) iR = 1
      if (realPart(iP).gt.lut_realPartsLUT(lut_numRealParts)) iR = lut_numRealParts - 1
    endif
    if (iR.eq.1) then
      x1 = lut_realPartsLUT(iR)
      x2 = lut_realPartsLUT(iR + 1)
      x3 = lut_realPartsLUT(iR + 2)
      realPartIndex(1) = iR
      realPartIndex(2) = iR + 1
      realPartIndex(3) = iR + 2
    else
      x1 = lut_realPartsLUT(iR - 1)
      x2 = lut_realPartsLUT(iR)
      x3 = lut_realPartsLUT(iR + 1)
      realPartIndex(1) = iR - 1
      realPartIndex(2) = iR
      realPartIndex(3) = iR + 1
    endif
    coefReal(1) = (realPart(iP) - x2)*(realPart(iP) - x3)/(x1 - x2)/(x1 - x3)
    coefReal(2) = (realPart(iP) - x1)*(realPart(iP) - x3)/(x2 - x1)/(x2 - x3)
    coefReal(3) = (realPart(iP) - x1)*(realPart(iP) - x2)/(x3 - x1)/(x3 - x2)

    lCRIFound = .false.
    do iI = 1, lut_numImagParts - 1
      if (imagPart(iP).ge.lut_imagPartsLUT(iI).and.imagPart(iP).le.lut_imagPartsLUT(iI + 1)) then
        lCRIFound = .true.
        exit
      endif
    enddo
    if (.not.lCRIFound) then
      if (imagPart(iP).lt.lut_imagPartsLUT(1)) iI = 1
      if (imagPart(iP).gt.lut_imagPartsLUT(lut_numImagParts)) iI = lut_numImagParts - 1
    endif
    if (iI.eq.1) then
      x1 = lut_imagPartsLUT(iI)
      x2 = lut_imagPartsLUT(iI + 1)
      x3 = lut_imagPartsLUT(iI + 2)
      imagPartIndex(1) = iI
      imagPartIndex(2) = iI + 1
      imagPartIndex(3) = iI + 2
    else
      x1 = lut_imagPartsLUT(iI - 1)
      x2 = lut_imagPartsLUT(iI)
      x3 = lut_imagPartsLUT(iI + 1)
      imagPartIndex(1) = iI - 1
      imagPartIndex(2) = iI
      imagPartIndex(3) = iI + 1
    endif
    coefImag(1) = (imagPart(iP) - x2)*(imagPart(iP) - x3)/(x1 - x2)/(x1 - x3)
    coefImag(2) = (imagPart(iP) - x1)*(imagPart(iP) - x3)/(x2 - x1)/(x2 - x3)
    coefImag(3) = (imagPart(iP) - x1)*(imagPart(iP) - x2)/(x3 - x1)/(x3 - x2)

    do iL = 1, lut_numLUT
      do j = 1, 3
        do k = 1, 3
          recordIndex = lut_numImagParts*(realPartIndex(j) - 1) + imagPartIndex(k)
          call sir_sca_get_data_lut(wavelength, iP, recordIndex, iL, iW, &
                                               extBuf, scaBuf, p11Buf, p12Buf, p22Buf, p33Buf, p34Buf, p44Buf)
          coefRI = lut_lutFraction(iL)*coefReal(j)*coefImag(k)
          lut_extCoef(iP, iW, iL) = lut_extCoef(iP, iW, iL) + extBuf*coefRI
          lut_scaCoef(iP, iW, iL) = lut_scaCoef(iP, iW, iL) + scaBuf*coefRI
          do iA = 1, lut_numScatteringAngles
            lut_p11(iA, iP, iW, iL) = lut_p11(iA, iP, iW, iL) + p11Buf(iA)*coefRI
            lut_p12(iA, iP, iW, iL) = lut_p12(iA, iP, iW, iL) + p12Buf(iA)*coefRI
            lut_p33(iA, iP, iW, iL) = lut_p33(iA, iP, iW, iL) + p33Buf(iA)*coefRI
            lut_p34(iA, iP, iW, iL) = lut_p34(iA, iP, iW, iL) + p34Buf(iA)*coefRI
          enddo
          if (lut_nonSpherical) then
            do iA = 1, lut_numScatteringAngles
              lut_p22(iA, iP, iW, iL) = lut_p22(iA, iP, iW, iL) + p22Buf(iA)*coefRI
              lut_p44(iA, iP, iW, iL) = lut_p44(iA, iP, iW, iL) + p44Buf(iA)*coefRI
            enddo
          endif
        enddo
      enddo
      lut_absCoef(iP, iW, iL) = abs(lut_extCoef(iP, iW, iL) - lut_scaCoef(iP, iW, iL))
      lut_ssa(iP, iW, iL) = lut_scaCoef(iP, iW, iL)/lut_extCoef(iP, iW, iL)
      lut_extCoefFinal(iW) = lut_extCoefFinal(iW) + lut_extCoef(iP, iW, iL)
      lut_scaCoefFinal(iW) = lut_scaCoefFinal(iW) + lut_scaCoef(iP, iW, iL)
      do iA = 1, lut_numScatteringAngles
        lut_p11Final(iA, iW) = lut_p11Final(iA, iW) + lut_p11(iA, iP, iW, iL)
        lut_p12Final(iA, iW) = lut_p12Final(iA, iW) + lut_p12(iA, iP, iW, iL)
        lut_p33Final(iA, iW) = lut_p33Final(iA, iW) + lut_p33(iA, iP, iW, iL)
        lut_p34Final(iA, iW) = lut_p34Final(iA, iW) + lut_p34(iA, iP, iW, iL)
      enddo
      if (lut_nonSpherical) then
        do iA = 1, lut_numScatteringAngles
          lut_p22Final(iA, iW) = lut_p22Final(iA, iW) + lut_p22(iA, iP, iW, iL)
          lut_p44Final(iA, iW) = lut_p44Final(iA, iW) + lut_p44(iA, iP, iW, iL)
        enddo
      endif
    enddo ! do iL = 1, lut_numLUT
  enddo ! do iP = 1, numberOfModes

  if (abs(nFinal).gt.0d0)  lut_GFinal = lut_GFinal/nFinal
  if (abs(sFinal).gt.0d0)  lut_reffFinal = vFinal/sFinal
  if (abs(sFinal).gt.0d0.and.abs(lut_reffFinal).gt.0d0) then
    lut_veffFinal = 1 + (final4 - 2d0*lut_reffFinal*vFinal)/sFinal/lut_reffFinal/lut_reffFinal
  endif

  if (lut_useNumerical) then
    hln = (log(lut_radiiGridBins(lut_numRadiiGridBins)) - log(lut_radiiGridBins(1)))/(lut_numRadiiGridBins - 1d0)
    bufN = 0d0
    bufS = 0d0
    bufV = 0d0
    buf4 = 0d0
    do iGB = 1, lut_numRadiiGridBins
      dVdln = 0d0
      do iP = 1, numberOfModes
        dVdln = dVdln + lut_dV_dlnr(iGB, iP)
      enddo
      bufN = bufN + 3d0*dVdln/4d0/lut_pi/lut_radiiGridBins(iGB)/lut_radiiGridBins(iGB)/lut_radiiGridBins(iGB)
      bufS = bufS + 3d0*dVdln/lut_radiiGridBins(iGB)
      bufV = bufV + dVdln
      buf4 = buf4 + 3d0*dVdln/4d0/lut_pi*lut_radiiGridBins(iGB)
    enddo
    bufN = bufN*hln
    bufS = bufS*hln
    bufV = bufV*hln
    buf4 = buf4*hln
    bufReff = 0d0
    if (abs(bufS).gt.0d0)  bufReff = 3d0*bufV/bufS
    bufG = 0d0
    if (abs(bufN).gt.0d0)  bufG = bufS/bufN/4d0
    bufVeff = 0d0
    if (abs(bufS).gt.0d0.and.abs(bufReff).gt.0d0)  bufVeff = 4d0*lut_pi*buf4/bufS/bufReff/bufReff - 1d0
    lut_GFinal = bufG
    lut_reffFinal = bufReff
    lut_veffFinal = bufVeff
  endif ! if (lut_useNumerical) then

  lut_absCoefFinal(iW) = abs(lut_extCoefFinal(iW) - lut_scaCoefFinal(iW))
  lut_ssaFinal(iW) = lut_scaCoefFinal(iW)/lut_extCoefFinal(iW)
  do iA = 1, lut_numScatteringAngles
    lut_p12Final(iA, iW) = lut_p12Final(iA, iW)/lut_p11Final(iA, iW)
    lut_p22Final(iA, iW) = lut_p22Final(iA, iW)/lut_p11Final(iA, iW)
    lut_p33Final(iA, iW) = lut_p33Final(iA, iW)/lut_p11Final(iA, iW)
    lut_p34Final(iA, iW) = lut_p34Final(iA, iW)/lut_p11Final(iA, iW)
    lut_p44Final(iA, iW) = lut_p44Final(iA, iW)/lut_p11Final(iA, iW)
    lut_p11Final(iA, iW) = lut_p11Final(iA, iW)/lut_scaCoefFinal(iW)
  enddo

  lut_asymmetryParamFinal(iW) = 0d0
  numberIntegrationAngles = 721
  do j = 1, numberIntegrationAngles
    curAngle = (j - 1)*180d0/(numberIntegrationAngles - 1)
    buf = 0d0
    do iA = 1, lut_numScatteringAngles - 1
      if (curAngle.ge.lut_scatteringAngles(iA).and.curAngle.le.lut_scatteringAngles(iA + 1)) then
        if (iA.lt.0.5d0*lut_numScatteringAngles) then
          x1 = lut_scatteringAngles(iA)
          y1 = log(lut_p11Final(iA, iW))
          x2 = lut_scatteringAngles(iA + 1)
          y2 = log(lut_p11Final(iA + 1, iW))
          x3 = lut_scatteringAngles(iA + 2)
          y3 = log(lut_p11Final(iA + 2, iW))
        else
          x1 = lut_scatteringAngles(iA - 1)
          y1 = log(lut_p11Final(iA - 1, iW))
          x2 = lut_scatteringAngles(iA)
          y2 = log(lut_p11Final(iA, iW))
          x3 = lut_scatteringAngles(iA + 1)
          y3 = log(lut_p11Final(iA + 1, iW))
        endif
        buf = y1*(curAngle - x2)*(curAngle - x3)/(x1 - x2)/(x1 - x3) + &
              y2*(curAngle - x1)*(curAngle - x3)/(x2 - x1)/(x2 - x3) + &
              y3*(curAngle - x1)*(curAngle - x2)/(x3 - x1)/(x3 - x2)
        curAngle = curAngle*lut_pi/180d0
        buf = exp(buf)*sin(curAngle)*cos(curAngle)
        exit
      endif
    enddo
    ! Numerical integration using Simpson's rule
    if (j.eq.1.or.j.eq.numberIntegrationAngles) then
      lut_asymmetryParamFinal(iW) = lut_asymmetryParamFinal(iW) + 0.5d0*buf
    else
      if (mod(j, 2).eq.1) then
        lut_asymmetryParamFinal(iW) = lut_asymmetryParamFinal(iW) + buf     ! odd
      else 
        lut_asymmetryParamFinal(iW) = lut_asymmetryParamFinal(iW) + 2d0*buf ! even
      endif
    endif
  enddo
  lut_asymmetryParamFinal(iW) = lut_pi/(numberIntegrationAngles - 1d0)/3d0*lut_asymmetryParamFinal(iW)

end subroutine sir_sca_driver