! Subroutine estimates parameters for the complex refractive index that is covered by look-up table
subroutine sir_sca_get_data_lut(wavelength, iP, recordIndex, iL, iW, &
                                extCoef, scaCoef, p11, p12, p22, p33, p34, p44)

  use sir_sca_store
  implicit none

  integer, intent(in out) :: iL, recordIndex
  integer, intent(in)     :: iP, iW
  real*8, intent(in)      :: wavelength
  real*8, intent(out)     :: extCoef, p11(500), p12(500), p22(500), p33(500), p34(500), p44(500), scaCoef

  integer iA, iGB, istat, j, k, l, lutIndexBuf, reclRI, sizeOfHeader, sizeOfRecord
  integer(kind = 8) curPos
  logical lOneByOne
  real*8  kernelExt, kernelSca, radius, wavelengthRatio, x1, x2, x3, y1, y2, y3
  real*8, allocatable, dimension(:) :: kernel11, kernel12, kernel22, kernel33, kernel34, kernel44

  extCoef = 0d0
  scaCoef = 0d0
  do iA = 1, lut_numScatteringAngles
    p11(iA) = 0d0
    p12(iA) = 0d0
    p22(iA) = 0d0
    p33(iA) = 0d0
    p34(iA) = 0d0
    p44(iA) = 0d0
  enddo

  inquire(iolength = reclRI) lut_wavelengthLUT

  allocate(kernel11(lut_numScatteringAngles), stat = istat)
  if (istat /= 0) stop "Not enough memory to allocate array. See sir_sca_get_data_lut.01"
  allocate(kernel12(lut_numScatteringAngles), stat = istat)
  if (istat /= 0) stop "Not enough memory to allocate array. See sir_sca_get_data_lut.02"
  allocate(kernel33(lut_numScatteringAngles), stat = istat)
  if (istat /= 0) stop "Not enough memory to allocate array. See sir_sca_get_data_lut.03"
  allocate(kernel34(lut_numScatteringAngles), stat = istat)
  if (istat /= 0) stop "Not enough memory to allocate array. See sir_sca_get_data_lut.04"
  if (lut_nonSpherical) then
    allocate(kernel22(lut_numScatteringAngles), stat = istat)
    if (istat /= 0) stop "Not enough memory to allocate array. See sir_sca_get_data_lut.05"
    allocate(kernel44(lut_numScatteringAngles), stat = istat)
    if (istat /= 0) stop "Not enough memory to allocate array. See sir_sca_get_data_lut.06"
  endif

  wavelengthRatio = lut_wavelengthLUT/wavelength

  lOneByOne = .true.
  if (allocated(lut_Cext)) lOneByOne = .false. ! Look-up table is uploaded into memory

  if (lOneByOne) then
    allocate(lut_realPartRecord(1), stat = istat)
    if (istat /= 0) stop "Not enough memory to allocate array. See sir_sca_get_data_lut.07"
    allocate(lut_imagPartRecord(1), stat = istat)
    if (istat /= 0) stop "Not enough memory to allocate array. See sir_sca_get_data_lut.08"
    allocate(lut_Cext(lut_numRadiiGridBins, 1, 1), stat = istat)
    if (istat /= 0) stop "Not enough memory to allocate array. See sir_sca_get_data_lut.09"
    allocate(lut_Csca(lut_numRadiiGridBins, 1, 1), stat = istat)
    if (istat /= 0) stop "Not enough memory to allocate array. See sir_sca_get_data_lut.10"
    allocate(lut_C11(lut_numScatteringAngles, lut_numRadiiGridBins, 1, 1), stat = istat)
    if (istat /= 0) stop "Not enough memory to allocate array. See sir_sca_get_data_lut.11"
    allocate(lut_C12(lut_numScatteringAngles, lut_numRadiiGridBins, 1, 1), stat = istat)
    if (istat /= 0) stop "Not enough memory to allocate array. See sir_sca_get_data_lut.12"
    allocate(lut_C33(lut_numScatteringAngles, lut_numRadiiGridBins, 1, 1), stat = istat)
    if (istat /= 0) stop "Not enough memory to allocate array. See sir_sca_get_data_lut.13"
    allocate(lut_C34(lut_numScatteringAngles, lut_numRadiiGridBins, 1, 1), stat = istat)
    if (istat /= 0) stop "Not enough memory to allocate array. See sir_sca_get_data_lut.14"
    if (lut_nonSpherical) then
      allocate(lut_C22(lut_numScatteringAngles, lut_numRadiiGridBins, 1, 1), stat = istat)
      if (istat /= 0) stop "Not enough memory to allocate array. See sir_sca_get_data_lut.15"
      allocate(lut_C44(lut_numScatteringAngles, lut_numRadiiGridBins, 1, 1), stat = istat)
      if (istat /= 0) stop "Not enough memory to allocate array. See sir_sca_get_data_lut.16"
    endif

    sizeOfHeader = 1 + 1 + lut_numRadiiGridBins + 1 + lut_numScatteringAngles + &
                   1 + lut_numRealParts + 1 + lut_numImagParts
    sizeOfRecord = 1 + 1 + lut_numRadiiGridBins + lut_numRadiiGridBins + &
                   4*lut_numRadiiGridBins*lut_numScatteringAngles
    if (lut_nonSpherical) then
      sizeOfHeader = 1 + 1 + 1 + lut_numRadiiGridBins + 1 + lut_numScatteringAngles + &
                     1 + lut_numRealParts + 1 + lut_numImagParts
      sizeOfRecord = 1 + 1 + lut_numRadiiGridBins + lut_numRadiiGridBins + &
                     6*lut_numRadiiGridBins*lut_numScatteringAngles
    endif

    open(unit = 33, file = lut_lutFileName(iL), access = 'direct', action = 'read', form = 'unformatted', recl = reclRI)
    curPos = 1 + sizeOfHeader + sizeOfRecord*(recordIndex - 1)
    recordIndex = 1
    lutIndexBuf = iL
    iL = 1
    read(33, rec = curPos) lut_realPartRecord(1)
    curPos = curPos + 1
    read(33, rec = curPos) lut_imagPartRecord(1)
    curPos = curPos + 1
    do j = 1, lut_numRadiiGridBins
      read(33, rec = curPos) lut_Cext(j, 1, 1)
      curPos = curPos + 1
    enddo
    do j = 1, lut_numRadiiGridBins
      read(33, rec = curPos) lut_Csca(j, 1, 1)
      curPos = curPos + 1
    enddo
    if (lut_nonSpherical) then
      do j = 1, lut_numRadiiGridBins
        lut_Csca(j, 1, 1) = lut_Cext(j, 1, 1) - lut_Csca(j, 1, 1)
      enddo
    endif
    do j = 1, lut_numRadiiGridBins
      do k = 1, lut_numScatteringAngles
        read(33, rec = curPos) lut_C11(k, j, 1, 1)
        curPos = curPos + 1
      enddo
    enddo
    do j = 1, lut_numRadiiGridBins
      do k = 1, lut_numScatteringAngles
        read(33, rec = curPos) lut_C12(k, j, 1, 1)
        curPos = curPos + 1
      enddo
    enddo
    if (lut_nonSpherical) then
      do j = 1, lut_numRadiiGridBins
        do k = 1, lut_numScatteringAngles
          read(33, rec = curPos) lut_C22(k, j, 1, 1)
          curPos = curPos + 1
        enddo
      enddo
    endif
    do j = 1, lut_numRadiiGridBins
      do k = 1, lut_numScatteringAngles
        read(33, rec = curPos) lut_C33(k, j, 1, 1)
        curPos = curPos + 1
      enddo
    enddo
    do j = 1, lut_numRadiiGridBins
      do k = 1, lut_numScatteringAngles
        read(33, rec = curPos) lut_C34(k, j, 1, 1)
        curPos = curPos + 1
      enddo
    enddo
    if (lut_nonSpherical) then
      do j = 1, lut_numRadiiGridBins
        do k = 1, lut_numScatteringAngles
          read(33, rec = curPos) lut_C44(k, j, 1, 1)
          curPos = curPos + 1
        enddo
      enddo
    endif
    close(33)
  endif

  k = 1
  do iGB = 1, lut_numRadiiGridBins
    radius = wavelengthRatio*lut_radiiGridBins(iGB)
    do l = k, lut_numRadiiGridBins - 1
      if (radius.ge.lut_radiiGridBins(l).and.radius.le.lut_radiiGridBins(l + 1)) then
        if (l.gt.1) then
          kernelExt = lut_Cext(l - 1, recordIndex, iL)*lut_coef1(iGB, iW) + &
                      lut_Cext(l, recordIndex, iL)*lut_coef2(iGB, iW) +     &
                      lut_Cext(l + 1, recordIndex, iL)*lut_coef3(iGB, iW)
          kernelSca = lut_Csca(l - 1, recordIndex, iL)*lut_coef1(iGB, iW) + &
                      lut_Csca(l, recordIndex, iL)*lut_coef2(iGB, iW) +     &
                      lut_Csca(l + 1, recordIndex, iL)*lut_coef3(iGB, iW)
          do iA = 1, lut_numScatteringAngles
            kernel11(iA) = lut_C11(iA, l - 1, recordIndex, iL)*lut_coef1(iGB, iW) + &
                           lut_C11(iA, l, recordIndex, iL)*lut_coef2(iGB, iW) +     &
                           lut_C11(iA, l + 1, recordIndex, iL)*lut_coef3(iGB, iW)
            kernel12(iA) = lut_C12(iA, l - 1, recordIndex, iL)*lut_coef1(iGB, iW) + &
                           lut_C12(iA, l, recordIndex, iL)*lut_coef2(iGB, iW) +     &
                           lut_C12(iA, l + 1, recordIndex, iL)*lut_coef3(iGB, iW)
            kernel33(iA) = lut_C33(iA, l - 1, recordIndex, iL)*lut_coef1(iGB, iW) + &
                           lut_C33(iA, l, recordIndex, iL)*lut_coef2(iGB, iW) +     &
                           lut_C33(iA, l + 1, recordIndex, iL)*lut_coef3(iGB, iW)
            kernel34(iA) = lut_C34(iA, l - 1, recordIndex, iL)*lut_coef1(iGB, iW) + &
                           lut_C34(iA, l, recordIndex, iL)*lut_coef2(iGB, iW) +     &
                           lut_C34(iA, l + 1, recordIndex, iL)*lut_coef3(iGB, iW)
          enddo
          if (lut_nonSpherical) then
            do iA = 1, lut_numScatteringAngles
              kernel22(iA) = lut_C22(iA, l - 1, recordIndex, iL)*lut_coef1(iGB, iW) + &
                             lut_C22(iA, l, recordIndex, iL)*lut_coef2(iGB, iW) +     &
                             lut_C22(iA, l + 1, recordIndex, iL)*lut_coef3(iGB, iW)
              kernel44(iA) = lut_C44(iA, l - 1, recordIndex, iL)*lut_coef1(iGB, iW) + &
                             lut_C44(iA, l, recordIndex, iL)*lut_coef2(iGB, iW) +     &
                             lut_C44(iA, l + 1, recordIndex, iL)*lut_coef3(iGB, iW)
            enddo
          endif
        else
          kernelExt = lut_Cext(l, recordIndex, iL)*lut_coef1(iGB, iW) +     &
                      lut_Cext(l + 1, recordIndex, iL)*lut_coef2(iGB, iW) + &
                      lut_Cext(l + 2, recordIndex, iL)*lut_coef3(iGB, iW)
          kernelSca = lut_Csca(l, recordIndex, iL)*lut_coef1(iGB, iW) +     &
                      lut_Csca(l + 1, recordIndex, iL)*lut_coef2(iGB, iW) + &
                      lut_Csca(l + 2, recordIndex, iL)*lut_coef3(iGB, iW)
          do iA = 1, lut_numScatteringAngles
            kernel11(iA) = lut_C11(iA, l, recordIndex, iL)*lut_coef1(iGB, iW) +     &
                           lut_C11(iA, l + 1, recordIndex, iL)*lut_coef2(iGB, iW) + &
                           lut_C11(iA, l + 2, recordIndex, iL)*lut_coef3(iGB, iW)
            kernel12(iA) = lut_C12(iA, l, recordIndex, iL)*lut_coef1(iGB, iW) +     &
                           lut_C12(iA, l + 1, recordIndex, iL)*lut_coef2(iGB, iW) + &
                           lut_C12(iA, l + 2, recordIndex, iL)*lut_coef3(iGB, iW)
            kernel33(iA) = lut_C33(iA, l, recordIndex, iL)*lut_coef1(iGB, iW) +     &
                           lut_C33(iA, l + 1, recordIndex, iL)*lut_coef2(iGB, iW) + &
                           lut_C33(iA, l + 2, recordIndex, iL)*lut_coef3(iGB, iW)
            kernel34(iA) = lut_C34(iA, l, recordIndex, iL)*lut_coef1(iGB, iW) +     &
                           lut_C34(iA, l + 1, recordIndex, iL)*lut_coef2(iGB, iW) + &
                           lut_C34(iA, l + 2, recordIndex, iL)*lut_coef3(iGB, iW)
          enddo
          if (lut_nonSpherical) then
            do iA = 1, lut_numScatteringAngles
              kernel22(iA) = lut_C22(iA, l, recordIndex, iL)*lut_coef1(iGB, iW) +     &
                             lut_C22(iA, l + 1, recordIndex, iL)*lut_coef2(iGB, iW) + &
                             lut_C22(iA, l + 2, recordIndex, iL)*lut_coef3(iGB, iW)
              kernel44(iA) = lut_C44(iA, l, recordIndex, iL)*lut_coef1(iGB, iW) +     &
                             lut_C44(iA, l + 1, recordIndex, iL)*lut_coef2(iGB, iW) + &
                             lut_C44(iA, l + 2, recordIndex, iL)*lut_coef3(iGB, iW)
            enddo
          endif
        endif
        extCoef = extCoef + kernelExt*lut_dV_dlnr(iGB, iP)
        scaCoef = scaCoef + kernelSca*lut_dV_dlnr(iGB, iP)
        do iA = 1, lut_numScatteringAngles
          p11(iA) = p11(iA) + kernel11(iA)*lut_dV_dlnr(iGB, iP)
          p12(iA) = p12(iA) + kernel12(iA)*lut_dV_dlnr(iGB, iP)
          p33(iA) = p33(iA) + kernel33(iA)*lut_dV_dlnr(iGB, iP)
          p34(iA) = p34(iA) + kernel34(iA)*lut_dV_dlnr(iGB, iP)
        enddo
        if (lut_nonSpherical) then
          do iA = 1, lut_numScatteringAngles
            p22(iA) = p22(iA) + kernel22(iA)*lut_dV_dlnr(iGB, iP)
            p44(iA) = p44(iA) + kernel44(iA)*lut_dV_dlnr(iGB, iP)
          enddo
        endif
        k = l
        exit
      endif
    enddo
  enddo ! do iGB = 1, lut_numRadiiGridBins

  extCoef = extCoef*wavelengthRatio
  scaCoef = scaCoef*wavelengthRatio
  do iA = 1, lut_numScatteringAngles
    p11(iA) = p11(iA)*wavelengthRatio
    p12(iA) = p12(iA)*wavelengthRatio
    p33(iA) = p33(iA)*wavelengthRatio
    p34(iA) = p34(iA)*wavelengthRatio
  enddo
  if (lut_nonSpherical) then
    do iA = 1, lut_numScatteringAngles
      p22(iA) = p22(iA)*wavelengthRatio
      p44(iA) = p44(iA)*wavelengthRatio
    enddo
  endif

  deallocate(kernel11)
  deallocate(kernel12)
  deallocate(kernel33)
  deallocate(kernel34)
  if (lut_nonSpherical) then
    deallocate(kernel22)
    deallocate(kernel44)
  endif

  if (lOneByOne) then
    iL = lutIndexBuf
    deallocate(lut_realPartRecord)
    deallocate(lut_imagPartRecord)
    deallocate(lut_Cext)
    deallocate(lut_Csca)
    deallocate(lut_C11)
    deallocate(lut_C12)
    deallocate(lut_C33)
    deallocate(lut_C34)
    if (lut_nonSpherical) then
      deallocate(lut_C22)
      deallocate(lut_C44)
    endif
  endif

end subroutine sir_sca_get_data_lut