! Subroutine reads coefficients from look-up table file into memory
subroutine sir_sca_read_lut_full

  use sir_sca_store
  implicit none

  integer iA, iC, iGB, iL, istat, numCRI, reclRI, sizeOfHeader
  integer(kind = 8) curPos

  inquire(iolength = reclRI) lut_wavelengthLUT

  call sir_sca_read_lut_quick

  numCRI = lut_numRealParts*lut_numImagParts

  if (allocated(lut_realPartRecord)) deallocate(lut_realPartRecord)
  allocate(lut_realPartRecord(numCRI), stat = istat)
  if (istat /= 0) stop "Not enough memory to allocate array. See sir_sca_read_lut_full.01"

  if (allocated(lut_imagPartRecord)) deallocate(lut_imagPartRecord)
  allocate(lut_imagPartRecord(numCRI), stat = istat)
  if (istat /= 0) stop "Not enough memory to allocate array. See sir_sca_read_lut_full.02"

  if (allocated(lut_Cext)) deallocate(lut_Cext)
  allocate(lut_Cext(lut_numRadiiGridBins, numCRI, lut_numLUT), stat = istat)
  if (istat /= 0) stop "Not enough memory to allocate array. See sir_sca_read_lut_full.03"

  if (allocated(lut_Csca)) deallocate(lut_Csca)
  allocate(lut_Csca(lut_numRadiiGridBins, numCRI, lut_numLUT), stat = istat)
  if (istat /= 0) stop "Not enough memory to allocate array. See sir_sca_read_lut_full.04"

  if (allocated(lut_C11)) deallocate(lut_C11)
  allocate(lut_C11(lut_numScatteringAngles, lut_numRadiiGridBins, numCRI, lut_numLUT), stat = istat)
  if (istat /= 0) stop "Not enough memory to allocate array. See sir_sca_read_lut_full.05"

  if (allocated(lut_C12)) deallocate(lut_C12)
  allocate(lut_C12(lut_numScatteringAngles, lut_numRadiiGridBins, numCRI, lut_numLUT), stat = istat)
  if (istat /= 0) stop "Not enough memory to allocate array. See sir_sca_read_lut_full.06"

  if (allocated(lut_C22)) deallocate(lut_C22)
  allocate(lut_C22(lut_numScatteringAngles, lut_numRadiiGridBins, numCRI, lut_numLUT), stat = istat)
  if (istat /= 0) stop "Not enough memory to allocate array. See sir_sca_read_lut_full.07"

  if (allocated(lut_C33)) deallocate(lut_C33)
  allocate(lut_C33(lut_numScatteringAngles, lut_numRadiiGridBins, numCRI, lut_numLUT), stat = istat)
  if (istat /= 0) stop "Not enough memory to allocate array. See sir_sca_read_lut_full.08"

  if (allocated(lut_C34)) deallocate(lut_C34)
  allocate(lut_C34(lut_numScatteringAngles, lut_numRadiiGridBins, numCRI, lut_numLUT), stat = istat)
  if (istat /= 0) stop "Not enough memory to allocate array. See sir_sca_read_lut_full.09"

  if (allocated(lut_C44)) deallocate(lut_C44)
  allocate(lut_C44(lut_numScatteringAngles, lut_numRadiiGridBins, numCRI, lut_numLUT), stat = istat)
  if (istat /= 0) stop "Not enough memory to allocate array. See sir_sca_read_lut_full.10"

  do iL = 1, lut_numLUT
    do iC = 1, numCRI
      lut_realPartRecord(iC) = 0d0
      lut_imagPartRecord(iC) = 0d0
      do iGB = 1, lut_numRadiiGridBins
        lut_Cext(iGB, iC, iL) = 0d0
        lut_Csca(iGB, iC, iL) = 0d0
        do iA = 1, lut_numScatteringAngles
          lut_C11(iA, iGB, iC, iL) = 0d0
          lut_C12(iA, iGB, iC, iL) = 0d0
          lut_C22(iA, iGB, iC, iL) = 0d0
          lut_C33(iA, iGB, iC, iL) = 0d0
          lut_C34(iA, iGB, iC, iL) = 0d0
          lut_C44(iA, iGB, iC, iL) = 0d0
        enddo
      enddo
    enddo
  enddo

  sizeOfHeader = 1 + 1 + lut_numRadiiGridBins + 1 + lut_numScatteringAngles + &
                 1 + lut_numRealParts + 1 + lut_numImagParts
  if (lut_nonSpherical)  sizeOfHeader = sizeOfHeader + 1

  do iL = 1, lut_numLUT
    open(unit = 33, file = lut_lutFileName(iL), access = 'direct', action = 'read', form = 'unformatted', recl = reclRI)
    curPos = 1 + sizeOfHeader
    do iC = 1, numCRI
      read(33, rec = curPos) lut_realPartRecord(iC)
      curPos = curPos + 1
      read(33, rec = curPos) lut_imagPartRecord(iC)
      curPos = curPos + 1
      do iGB = 1, lut_numRadiiGridBins
        read(33, rec = curPos) lut_Cext(iGB, iC, iL)
        curPos = curPos + 1
      enddo
      do iGB = 1, lut_numRadiiGridBins
        read(33, rec = curPos) lut_Csca(iGB, iC, iL)
        curPos = curPos + 1
      enddo
      if (lut_nonSpherical) then
        do iGB = 1, lut_numRadiiGridBins
          lut_Csca(iGB, iC, iL) = lut_Cext(iGB, iC, iL) - lut_Csca(iGB, iC, iL)
        enddo
      endif
      do iGB = 1, lut_numRadiiGridBins
        do iA = 1, lut_numScatteringAngles
          read(33, rec = curPos) lut_C11(iA, iGB, iC, iL)
          curPos = curPos + 1
        enddo
      enddo
      do iGB = 1, lut_numRadiiGridBins
        do iA = 1, lut_numScatteringAngles
          read(33, rec = curPos) lut_C12(iA, iGB, iC, iL)
          curPos = curPos + 1
        enddo
      enddo
      if (lut_nonSpherical) then
        do iGB = 1, lut_numRadiiGridBins
          do iA = 1, lut_numScatteringAngles
            read(33, rec = curPos) lut_C22(iA, iGB, iC, iL)
            curPos = curPos + 1
          enddo
        enddo
      endif
      do iGB = 1, lut_numRadiiGridBins
        do iA = 1, lut_numScatteringAngles
          read(33, rec = curPos) lut_C33(iA, iGB, iC, iL)
          curPos = curPos + 1
        enddo
      enddo
      do iGB = 1, lut_numRadiiGridBins
        do iA = 1, lut_numScatteringAngles
          read(33, rec = curPos) lut_C34(iA, iGB, iC, iL)
          curPos = curPos + 1
        enddo
      enddo
      if (lut_nonSpherical) then
        do iGB = 1, lut_numRadiiGridBins
          do iA = 1, lut_numScatteringAngles
            read(33, rec = curPos) lut_C44(iA, iGB, iC, iL)
            curPos = curPos + 1
          enddo
        enddo
      endif
    enddo
    close(33)
  enddo

end subroutine sir_sca_read_lut_full