! Program uses look-up table to compute the elements of the normalized scattering matrix as well as the asymmetry parameter,
! the aerosol absorption, backscatter, extinction, and scattering coefficients
program sir_sca_usage

  use sir_sca_store
  implicit none

  character(100) outputName
  integer        i, iA, ndistr, nlambda, numberOfModes, wavelengthIndex
  real*8         countMedianRadius(4), imagPart(4), lnSigma(4), numberConcentration(4), realPart(4), wavelength(11)

  lut_pi = 3.1415926535897932384626433832795d0

  lut_lutFileName(1) = 'AerosolLUT_1000_100_0.355_650bins_2325CRI_ln2rKr_Twomey.dat'
  lut_lutFraction(1) = 1d0
  lut_nonSpherical = .false.
  lut_numLUT = 1

  ! Bimodal   aerosol if numberOfModes = 2
  ! Monomodal aerosol if numberOfModes = 1
  numberOfModes = 2

  ! Complex refractive index
  realPart(1) = 1.55d0
  realPart(2) = 1.52d0
  imagPart(1) = 0.04d0
  imagPart(2) = 0.0019d0

  ! Lognormal PSD
  ndistr = 2
  countMedianRadius(1) = 0.12d0
  countMedianRadius(2) = 0.5d0
  lnSigma(1) = 0.37d0
  lnSigma(2) = 0.75d0
  numberConcentration(1) = 10.0d0
  numberConcentration(2) = 5.0d0

  ! The wavelengths of interest
  nlambda = 5
  wavelength(1) = 0.355d0
  wavelength(2) = 0.410d0
  wavelength(3) = 0.469d0
  wavelength(4) = 0.550d0
  wavelength(5) = 0.865d0

!  call sir_sca_read_lut_quick
  call sir_sca_read_lut_full
  do wavelengthIndex = 1, nlambda
    call sir_sca_get_coef(wavelength(wavelengthIndex), wavelengthIndex, 0d0, 1000d0)
  enddo

  do wavelengthIndex = 1, nlambda
    call sir_sca_driver(wavelength(wavelengthIndex), wavelengthIndex, realPart, imagPart, &
                        ndistr, numberOfModes, countMedianRadius, lnSigma, numberConcentration)

    write(outputName, 1) wavelength(wavelengthIndex)
1   format('Output_', f14.12, '.txt')
    open(55, file = outputName)
    write(55, 2) wavelength(wavelengthIndex)
2   format('Wavelength = ', f14.12, ' mum')
    write(55, 3)
3   format('')
    do i = 1, numberOfModes
      write(55, 4) i
4     format('Aerosol mode ', i1)
      write(55, 5)
5     format('  Complex refractive index:')
      write(55, 6) realPart(i)
6     format('    real part = ', f14.12)
      write(55, 7) imagPart(i)
7     format('    imaginary part = ', f14.12)
      if (ndistr.eq.2) then
        write(55, 8)
8       format('  Logarithmic-normal particle size distribution:')
        write(55, 9) numberConcentration(i)
9       format('    total number concentration =', e19.12, ' 1/cm^3')
        write(55, 10) countMedianRadius(i)
10      format('    count median radius = ', f14.12, ' mum')
        write(55, 11) exp(lnsigma(i))
11      format('    geometric standard deviation = ', f14.12)
        write(55, 12) lut_G(i)
12      format('    <g> =', e19.12, ' mum^2')
        write(55, 13) lut_reff(i)
13      format('    effective radius =', e19.12, ' mum')
        write(55, 14) lut_veff(i)
14      format('    effective variance =', e19.12)
      endif
    enddo
    write(55, 15)
15  format('')
    write(55, 16)
16  format('Optical coefficients:')
    write(55, 17) lut_absCoefFinal(wavelengthIndex)
17  format('  absorption     =', e19.12, ' 1/Mm')
    write(55, 18) lut_extCoefFinal(wavelengthIndex)
18  format('  extinction     =', e19.12, ' 1/Mm')
    write(55, 19) lut_scaCoefFinal(wavelengthIndex)
19  format('  scattering     =', e19.12, ' 1/Mm')
    write(55, 20) lut_p11Final(lut_numScatteringAngles, wavelengthIndex)* &
                  lut_scaCoefFinal(wavelengthIndex)/4d0/lut_pi
20  format('  backscattering =', e19.12, ' 1/(Mm sr)')
    write(55, 21) lut_ssaFinal(wavelengthIndex)
21  format('Single-scattering albedo = ', f14.12)
    write(55, 22) lut_asymmetryParamFinal(wavelengthIndex)
22  format('Asymmetry parameter =', e19.12)
    write(55, 23) lut_GFinal
23  format('<G> =', e19.12, ' mum^2')
    write(55, 24) lut_reffFinal
24  format('Effective radius =', e19.12, ' mum')
    write(55, 25) lut_veffFinal
25  format('Effective variance =', e19.12)
    write(55, 26)
26  format('')
    if (lut_nonSpherical) then
      write(55, 27)
27    format('                       ******* Normalized scattering matrix *******')
      write(55, 28)
28    format('Angle  P11           P12/P11       P22/P11       P33/P11       P34/P11       P44/P11')
      do iA = 1, lut_numScatteringAngles
        write(55, 29) lut_scatteringAngles(iA), &
                      lut_p11Final(iA, wavelengthIndex), lut_p12Final(iA, wavelengthIndex), &
                      lut_p22Final(iA, wavelengthIndex), lut_p33Final(iA, wavelengthIndex), &
                      lut_p34Final(iA, wavelengthIndex), lut_p44Final(iA, wavelengthIndex)
29      format(f5.1, ' ', e13.6, ' ', e13.6, ' ', e13.6, ' ', e13.6, ' ', e13.6, ' ', e13.6)
      enddo
    else
      write(55, 30)
30    format('            ******* Normalized scattering matrix *******')
      write(55, 31)
31    format('Angle  P11           P12/P11       P33/P11       P34/P11')
      do iA = 1, lut_numScatteringAngles
        write(55, 32) lut_scatteringAngles(iA), &
                      lut_p11Final(iA, wavelengthIndex), lut_p12Final(iA, wavelengthIndex), &
                      lut_p33Final(iA, wavelengthIndex), lut_p34Final(iA, wavelengthIndex)
32      format(f5.1, ' ', e13.6, ' ', e13.6, ' ', e13.6, ' ', e13.6)
      enddo
    endif
    close(55)
  enddo ! do wavelengthIndex = 1, nlambda

  if (allocated(lut_radiiGridBins))    deallocate(lut_radiiGridBins)
  if (allocated(lut_scatteringAngles)) deallocate(lut_scatteringAngles)
  if (allocated(lut_realPartsLUT))     deallocate(lut_realPartsLUT)
  if (allocated(lut_imagPartsLUT))     deallocate(lut_imagPartsLUT)
  if (allocated(lut_realPartRecord))   deallocate(lut_realPartRecord)
  if (allocated(lut_imagPartRecord))   deallocate(lut_imagPartRecord)
  if (allocated(lut_Cext))             deallocate(lut_Cext)
  if (allocated(lut_Csca))             deallocate(lut_Csca)
  if (allocated(lut_C11))              deallocate(lut_C11)
  if (allocated(lut_C12))              deallocate(lut_C12)
  if (allocated(lut_C22))              deallocate(lut_C22)
  if (allocated(lut_C33))              deallocate(lut_C33)
  if (allocated(lut_C34))              deallocate(lut_C34)
  if (allocated(lut_C44))              deallocate(lut_C44)

  if (allocated(lut_absCoef))             deallocate(lut_absCoef)
  if (allocated(lut_absCoefFinal))        deallocate(lut_absCoefFinal)
  if (allocated(lut_asymmetryParamFinal)) deallocate(lut_asymmetryParamFinal)
  if (allocated(lut_coef1))               deallocate(lut_coef1)
  if (allocated(lut_coef2))               deallocate(lut_coef2)
  if (allocated(lut_coef3))               deallocate(lut_coef3)
  if (allocated(lut_dV_dlnr))             deallocate(lut_dV_dlnr)
  if (allocated(lut_extCoef))             deallocate(lut_extCoef)
  if (allocated(lut_extCoefFinal))        deallocate(lut_extCoefFinal)
  if (allocated(lut_scaCoef))             deallocate(lut_scaCoef)
  if (allocated(lut_scaCoefFinal))        deallocate(lut_scaCoefFinal)
  if (allocated(lut_ssa))                 deallocate(lut_ssa)
  if (allocated(lut_ssaFinal))            deallocate(lut_ssaFinal)
  if (allocated(lut_p11))                 deallocate(lut_p11)
  if (allocated(lut_p11Final))            deallocate(lut_p11Final)
  if (allocated(lut_p12))                 deallocate(lut_p12)
  if (allocated(lut_p12Final))            deallocate(lut_p12Final)
  if (allocated(lut_p22))                 deallocate(lut_p22)
  if (allocated(lut_p22Final))            deallocate(lut_p22Final)
  if (allocated(lut_p33))                 deallocate(lut_p33)
  if (allocated(lut_p33Final))            deallocate(lut_p33Final)
  if (allocated(lut_p34))                 deallocate(lut_p34)
  if (allocated(lut_p34Final))            deallocate(lut_p34Final)
  if (allocated(lut_p44))                 deallocate(lut_p44)
  if (allocated(lut_p44Final))            deallocate(lut_p44Final)

  if (allocated(lut_G))                   deallocate(lut_G)
  if (allocated(lut_reff))                deallocate(lut_reff)
  if (allocated(lut_veff))                deallocate(lut_veff)

end program sir_sca_usage