function [bscCoef, extCoef, scaCoef, p11, p12, p22, p33, p34, p44] = GetData(mode, LUT, recordIndex, coef1, coef2, coef3, radiusIndexMin)
% "mode" defines the list of estimated parameters
% 1 - bscCoef, extCoef, scaCoef
% 2 - bscCoef, extCoef, scaCoef, p11, p12, p33, p34
% 3 - bscCoef, extCoef, scaCoef, p11, p12, p22, p33, p34, p44
  Cbsc = LUT.DataRecord(recordIndex).Cbsc;
  Cext = LUT.DataRecord(recordIndex).Cext;
  Csca = LUT.DataRecord(recordIndex).Csca;
  bscCoef = Cbsc(1)*coef1(radiusIndexMin) + Cbsc(2)*coef2(radiusIndexMin) + Cbsc(3)*coef3(radiusIndexMin);
  extCoef = Cext(1)*coef1(radiusIndexMin) + Cext(2)*coef2(radiusIndexMin) + Cext(3)*coef3(radiusIndexMin);
  scaCoef = Csca(1)*coef1(radiusIndexMin) + Csca(2)*coef2(radiusIndexMin) + Csca(3)*coef3(radiusIndexMin);
  indexMin = radiusIndexMin + 1;
  if indexMin == 2 % Requested wavelength is around wavelength of LUT itself
    bscCoef = 2*bscCoef;        extCoef = 2*extCoef;        scaCoef = 2*scaCoef;
    indexMin = indexMin + 1;
  end
  numRad = LUT.NumberRadiiGridBins - indexMin;
  bufCoef1 = coef1(indexMin : indexMin + numRad);
  bufCoef2 = coef2(indexMin : indexMin + numRad);
  bufCoef3 = coef3(indexMin : indexMin + numRad);
  bscCoef = bscCoef + sum(Cbsc(1 : 1 + numRad).*bufCoef1 + Cbsc(2 : 2 + numRad).*bufCoef2 + Cbsc(3 : 3 + numRad).*bufCoef3);
  extCoef = extCoef + sum(Cext(1 : 1 + numRad).*bufCoef1 + Cext(2 : 2 + numRad).*bufCoef2 + Cext(3 : 3 + numRad).*bufCoef3);
  scaCoef = scaCoef + sum(Csca(1 : 1 + numRad).*bufCoef1 + Csca(2 : 2 + numRad).*bufCoef2 + Csca(3 : 3 + numRad).*bufCoef3);

  if mode > 1
    C11  = LUT.DataRecord(recordIndex).C11;
    C12  = LUT.DataRecord(recordIndex).C12;
    C33  = LUT.DataRecord(recordIndex).C33;
    C34  = LUT.DataRecord(recordIndex).C34;
    p11 = C11(:, 1)*coef1(radiusIndexMin) + C11(:, 2)*coef2(radiusIndexMin) + C11(:, 3)*coef3(radiusIndexMin);
    p12 = C12(:, 1)*coef1(radiusIndexMin) + C12(:, 2)*coef2(radiusIndexMin) + C12(:, 3)*coef3(radiusIndexMin);
    p33 = C33(:, 1)*coef1(radiusIndexMin) + C33(:, 2)*coef2(radiusIndexMin) + C33(:, 3)*coef3(radiusIndexMin);
    p34 = C34(:, 1)*coef1(radiusIndexMin) + C34(:, 2)*coef2(radiusIndexMin) + C34(:, 3)*coef3(radiusIndexMin);
    if indexMin == 2 % Requested wavelength is around wavelength of LUT itself
      p11 = 2*p11;  p12 = 2*p12;  p33 = 2*p33;  p34 = 2*p34;
    end
    p11 = p11 + sum(C11(:, 1 : 1 + numRad).*bufCoef1 + C11(:, 2 : 2 + numRad).*bufCoef2 + C11(:, 3 : 3 + numRad).*bufCoef3, 2);
    p12 = p12 + sum(C12(:, 1 : 1 + numRad).*bufCoef1 + C12(:, 2 : 2 + numRad).*bufCoef2 + C12(:, 3 : 3 + numRad).*bufCoef3, 2);
    p33 = p33 + sum(C33(:, 1 : 1 + numRad).*bufCoef1 + C33(:, 2 : 2 + numRad).*bufCoef2 + C33(:, 3 : 3 + numRad).*bufCoef3, 2);
    p34 = p34 + sum(C34(:, 1 : 1 + numRad).*bufCoef1 + C34(:, 2 : 2 + numRad).*bufCoef2 + C34(:, 3 : 3 + numRad).*bufCoef3, 2);
  else
    p11 = 0;  p12 = 0;  p33 = 0;  p34 = 0;
  end

  if mode > 2
    C22  = LUT.DataRecord(recordIndex).C22;
    C44  = LUT.DataRecord(recordIndex).C44;
    p22 = C22(:, 1)*coef1(radiusIndexMin) + C22(:, 2)*coef2(radiusIndexMin) + C22(:, 3)*coef3(radiusIndexMin);
    p44 = C44(:, 1)*coef1(radiusIndexMin) + C44(:, 2)*coef2(radiusIndexMin) + C44(:, 3)*coef3(radiusIndexMin);
    if indexMin == 2 % Requested wavelength is around wavelength of LUT itself
      p22 = 2*p22;  p44 = 2*p44;
    end
    p22 = p22 + sum(C22(:, 1 : 1 + numRad).*bufCoef1 + C22(:, 2 : 2 + numRad).*bufCoef2 + C22(:, 3 : 3 + numRad).*bufCoef3, 2);
    p44 = p44 + sum(C44(:, 1 : 1 + numRad).*bufCoef1 + C44(:, 2 : 2 + numRad).*bufCoef2 + C44(:, 3 : 3 + numRad).*bufCoef3, 2);
  else
    p22 = 0;  p44 = 0;
  end
end