clearvars -except AerosolLUT
if ~exist('AerosolLUT', 'var')
  load('AerosolLUT_1000_100_0.355_650bins_2325CRI_ln2rKr_Twomey.mat');
end

% Change the desired wavelength (mum)
wavelength = 0.55;

% Bimodal   aerosol if numberOfModes = 2
% Monomodal aerosol if numberOfModes = 1
numberOfModes = 2;

% Complex refractive index
realPart(1) = 1.55;
realPart(2) = 1.52;
imagPart(1) = 0.04;
imagPart(2) = 0.0019;

% Lognormal PSD
countMedianRadius(1) = 0.12;
countMedianRadius(2) = 0.5;
lnSigma(1) = 0.37;
lnSigma(2) = 0.75;
numberConcentration(1) = 10;
numberConcentration(2) = 5;

% Read the header
wavelengthLUT          = AerosolLUT.WavelengthLUT;
numberRadiiGridBins    = AerosolLUT.NumberRadiiGridBins;
radiiGridBins          = AerosolLUT.RadiiGridBins;
numberScatteringAngles = AerosolLUT.NumberScatteringAngles;
scatteringAngles       = AerosolLUT.ScatteringAngles;
numberRealParts        = AerosolLUT.NumberRealParts;
realPartsCRI           = AerosolLUT.RealPartsCRI;
numberImagParts        = AerosolLUT.NumberImagParts;
imagPartsCRI           = AerosolLUT.ImagPartsCRI;

wavelengthRatio = wavelengthLUT/wavelength;
[coef1, coef2, coef3, radiusIndexMin] = GetCoef(radiiGridBins, 0, 1000, wavelengthRatio);

bscCoefFinal = 0;
extCoefFinal = 0;
scaCoefFinal = 0;
p11Final = zeros(1, numberScatteringAngles);
p12Final = zeros(1, numberScatteringAngles);
p33Final = zeros(1, numberScatteringAngles);
p34Final = zeros(1, numberScatteringAngles);

absCoef = zeros(1, numberOfModes);
bscCoef = zeros(1, numberOfModes);
extCoef = zeros(1, numberOfModes);
scaCoef = zeros(1, numberOfModes);
ssa     = zeros(1, numberOfModes);
p11     = zeros(numberOfModes, numberScatteringAngles);
p12     = zeros(numberOfModes, numberScatteringAngles);
p33     = zeros(numberOfModes, numberScatteringAngles);
p34     = zeros(numberOfModes, numberScatteringAngles);

for i = 1 : numberOfModes
  [l0a, l1a, l2a, ixa] = quad_interpolator_coefficients(realPartsCRI, realPart(i));
  [l0b, l1b, l2b, ixb] = quad_interpolator_coefficients(imagPartsCRI, imagPart(i));

  bscCoefArray = zeros(numberRealParts, numberImagParts);
  extCoefArray = zeros(numberRealParts, numberImagParts);
  scaCoefArray = zeros(numberRealParts, numberImagParts);
  p11Array = zeros(numberRealParts, numberImagParts, numberScatteringAngles);
  p12Array = zeros(numberRealParts, numberImagParts, numberScatteringAngles);
  p22Array = zeros(numberRealParts, numberImagParts, numberScatteringAngles);
  p33Array = zeros(numberRealParts, numberImagParts, numberScatteringAngles);
  p34Array = zeros(numberRealParts, numberImagParts, numberScatteringAngles);
  p44Array = zeros(numberRealParts, numberImagParts, numberScatteringAngles);

  dV_dlnr = zeros(1, numberRadiiGridBins);
  for iGB = 1 : numberRadiiGridBins % Lognormal PSD
    dV_dlnr(iGB) = 4*pi/3.*radiiGridBins(iGB)*radiiGridBins(iGB)*radiiGridBins(iGB) * numberConcentration(i)/sqrt(2*pi)/lnSigma(i)*exp(-(log(radiiGridBins(iGB)) - log(countMedianRadius(i)))*(log(radiiGridBins(iGB)) - log(countMedianRadius(i)))/2/lnSigma(i)/lnSigma(i));
  end

  coef1PSD = wavelengthRatio*coef1.*dV_dlnr;
  coef2PSD = wavelengthRatio*coef2.*dV_dlnr;
  coef3PSD = wavelengthRatio*coef3.*dV_dlnr;
  for iA = 1 : 3
    for iB = 1 : 3
      recordIndex = AerosolLUT.NumberImagParts*(ixa(iA) - 1) + ixb(iB);
      [bscCoefArray(ixa(iA), ixb(iB)), extCoefArray(ixa(iA), ixb(iB)), scaCoefArray(ixa(iA), ixb(iB)), p11Array(ixa(iA), ixb(iB), :), p12Array(ixa(iA), ixb(iB), :), p22Array(ixa(iA), ixb(iB), :), p33Array(ixa(iA), ixb(iB), :), p34Array(ixa(iA), ixb(iB), :), p44Array(ixa(iA), ixb(iB), :)] = GetData(2, AerosolLUT, recordIndex, coef1PSD, coef2PSD, coef3PSD, radiusIndexMin);
    end
  end
  tmp1(:, :) = bscCoefArray(ixa, ixb);
  tmp2(:)    = tmp1(:, 1)*l0b + tmp1(:, 2)*l1b + tmp1(:, 3)*l2b;
  bscCoef(i) = tmp2(1)*l0a + tmp2(2)*l1a + tmp2(3)*l2a;

  tmp1(:, :) = extCoefArray(ixa, ixb);
  tmp2(:)    = tmp1(:, 1)*l0b + tmp1(:, 2)*l1b + tmp1(:, 3)*l2b;
  extCoef(i) = tmp2(1)*l0a + tmp2(2)*l1a + tmp2(3)*l2a;

  tmp1(:, :) = scaCoefArray(ixa, ixb);
  tmp2(:)    = tmp1(:, 1)*l0b + tmp1(:, 2)*l1b + tmp1(:, 3)*l2b;
  scaCoef(i) = tmp2(1)*l0a + tmp2(2)*l1a + tmp2(3)*l2a;

  absCoef(i) = abs(extCoef(i) - scaCoef(i));
  ssa(i)     = scaCoef(i)/extCoef(i);

  tmp3(:, :, :) = p11Array(ixa, ixb, :);
  tmp4(:, :)    = tmp3(:, 1, :)*l0b + tmp3(:, 2, :)*l1b + tmp3(:, 3, :)*l2b;
  p11(i, :)     = tmp4(1, :)*l0a + tmp4(2, :)*l1a + tmp4(3, :)*l2a;

  tmp3(:, :, :) = p12Array(ixa, ixb, :);
  tmp4(:, :)    = tmp3(:, 1, :)*l0b + tmp3(:, 2, :)*l1b + tmp3(:, 3, :)*l2b;
  p12(i, :)     = tmp4(1, :)*l0a + tmp4(2, :)*l1a + tmp4(3, :)*l2a;

  tmp3(:, :, :) = p33Array(ixa, ixb, :);
  tmp4(:, :)    = tmp3(:, 1, :)*l0b + tmp3(:, 2, :)*l1b + tmp3(:, 3, :)*l2b;
  p33(i, :)     = tmp4(1, :)*l0a + tmp4(2, :)*l1a + tmp4(3, :)*l2a;

  tmp3(:, :, :) = p34Array(ixa, ixb, :);
  tmp4(:, :)    = tmp3(:, 1, :)*l0b + tmp3(:, 2, :)*l1b + tmp3(:, 3, :)*l2b;
  p34(i, :)     = tmp4(1, :)*l0a + tmp4(2, :)*l1a + tmp4(3, :)*l2a;

  bscCoefFinal = bscCoefFinal + bscCoef(i);
  extCoefFinal = extCoefFinal + extCoef(i);
  scaCoefFinal = scaCoefFinal + scaCoef(i);

  p11Final = p11Final + p11(i, :);
  p12Final = p12Final + p12(i, :);
  p33Final = p33Final + p33(i, :);
  p34Final = p34Final + p34(i, :);
end

absCoefFinal = abs(extCoefFinal - scaCoefFinal);
ssaFinal     = scaCoefFinal/extCoefFinal;
p11Final = p11Final./scaCoefFinal;
p12Final = p12Final./scaCoefFinal;
p33Final = p33Final./scaCoefFinal;
p34Final = p34Final./scaCoefFinal;

p12Final = p12Final./p11Final;
p33Final = p33Final./p11Final;
p34Final = p34Final./p11Final;

% Compute asymmetry parameter
asymmetryParamFinal = 0;
numberIntegrationAngles = 721;
for j = 1 : numberIntegrationAngles
  currentAngle = (j - 1)*180/(numberIntegrationAngles - 1);
  % Quadratic interpolation of p11 for the "currentAngle"
  bufValue = 0;
  for m = 1 : numberScatteringAngles - 1
    if currentAngle >= scatteringAngles(m) && currentAngle <= scatteringAngles(m + 1)
      if m < 0.5*numberScatteringAngles
        x1 = scatteringAngles(m);		y1 = log(p11Final(m));
        x2 = scatteringAngles(m + 1);	y2 = log(p11Final(m + 1));
        x3 = scatteringAngles(m + 2);	y3 = log(p11Final(m + 2));
      else
        x1 = scatteringAngles(m - 1);	y1 = log(p11Final(m - 1));
        x2 = scatteringAngles(m);		y2 = log(p11Final(m));
        x3 = scatteringAngles(m + 1);	y3 = log(p11Final(m + 1));
      end
      bufValue = exp(y1*(currentAngle - x2)*(currentAngle - x3)/(x1 - x2)/(x1 - x3) + y2*(currentAngle - x1)*(currentAngle - x3)/(x2 - x1)/(x2 - x3) + y3*(currentAngle - x1)*(currentAngle - x2)/(x3 - x1)/(x3 - x2));
      bufValue = bufValue*sin(currentAngle*pi/180)*cos(currentAngle*pi/180);
      break;
    end
  end
  % Numerical integration using Simpson's rule
  if j == 1 || j == numberIntegrationAngles
    asymmetryParamFinal = asymmetryParamFinal + 0.5*bufValue;
  else
    if bitget(j, 1) % odd
      asymmetryParamFinal = asymmetryParamFinal + bufValue;
    else % even
      asymmetryParamFinal = asymmetryParamFinal + 2*bufValue;
    end
  end
end
asymmetryParamFinal = pi/(numberIntegrationAngles - 1)/3 * asymmetryParamFinal;

fileName = num2str(wavelength, 'Output_%14.12f.txt');
fileID = fopen(fileName, 'w');
fprintf(fileID, 'Wavelength = %14.12f mum\n\n', wavelength);
gFinal = 0;  nFinal = 0;     sFinal = 0;  vFinal = 0;
final4 = 0;  reffFinal = 0;  veffFinal = 0;
for i = 1 : numberOfModes
  fprintf(fileID, 'Aerosol mode %d\n', i);
  fprintf(fileID, '  Complex refractive index:\n');
  fprintf(fileID, '    real part = %14.12f\n', realPart(i));
  fprintf(fileID, '    imaginary part = %14.12f\n', imagPart(i));
  fprintf(fileID, '  Logarithmic-normal particle size distribution:\n');
  fprintf(fileID, '    total number concentration =%19.12e 1/cm^3\n', numberConcentration(i));
  fprintf(fileID, '    count median radius = %14.12f mum\n', countMedianRadius(i));
  fprintf(fileID, '    geometric standard deviation = %14.12f\n', exp(lnSigma(i)));
  fprintf(fileID, '    <g> =%19.12e mum^2\n', pi*countMedianRadius(i)*countMedianRadius(i)*exp(2*lnSigma(i)*lnSigma(i)));
  fprintf(fileID, '    effective radius =%19.12e mum\n', countMedianRadius(i)*exp(2.5*lnSigma(i)*lnSigma(i)));
  fprintf(fileID, '    effective variance =%19.12e\n', exp(lnSigma(i)*lnSigma(i)) - 1);
  gFinal = gFinal + numberConcentration(i)*pi*countMedianRadius(i)*countMedianRadius(i)*exp(2*lnSigma(i)*lnSigma(i));
  nFinal = nFinal + numberConcentration(i);
  sFinal = sFinal + numberConcentration(i)*countMedianRadius(i)*countMedianRadius(i)*exp(2*lnSigma(i)*lnSigma(i));
  vFinal = vFinal + numberConcentration(i)*countMedianRadius(i)*countMedianRadius(i)*countMedianRadius(i)*exp(4.5*lnSigma(i)*lnSigma(i));
  final4 = final4 + numberConcentration(i)*countMedianRadius(i)*countMedianRadius(i)*countMedianRadius(i)*countMedianRadius(i)*exp(8*lnSigma(i)*lnSigma(i));
end
if nFinal > 0
  gFinal = gFinal/nFinal;
end
if sFinal > 0
  reffFinal = vFinal/sFinal;
end
if sFinal > 0 && reffFinal > 0
  veffFinal = 1 + (final4 - 2d0*reffFinal*vFinal)/sFinal/reffFinal/reffFinal;
end
fprintf(fileID, '\nOptical coefficients:\n');
fprintf(fileID, '  absorption     =%19.12e 1/Mm\n', absCoefFinal);
fprintf(fileID, '  extinction     =%19.12e 1/Mm\n', extCoefFinal);
fprintf(fileID, '  scattering     =%19.12e 1/Mm\n', scaCoefFinal);
fprintf(fileID, '  backscattering =%19.12e 1/(Mm sr)\n', bscCoefFinal);
fprintf(fileID, 'Single-scattering albedo = %14.12f\n', ssaFinal);
fprintf(fileID, 'Asymmetry parameter =%19.12e\n', asymmetryParamFinal);
fprintf(fileID, '<G> =%19.12e mum^2\n', gFinal);
fprintf(fileID, 'Effective radius =%19.12e mum\n', reffFinal);
fprintf(fileID, 'Effective variance =%19.12e\n\n', veffFinal);
fprintf(fileID, '            ******* Normalized scattering matrix *******\n');
fprintf(fileID, 'Angle  P11           P12/P11       P33/P11       P34/P11\n');
for m = 1 : numberScatteringAngles
  fprintf(fileID, '%5.1f %13.6e %13.6e %13.6e %13.6e\n', scatteringAngles(m), p11Final(m), p12Final(m), p33Final(m), p34Final(m));
end
fclose(fileID);