#!/usr/bin/env python
#/usr/local/pkgs-modules/Python_2.7.8/bin/python
# ^^ K path
#/opt/local/Library/Frameworks/Python.framework/Versions/2.7/bin/python2.7
# ^^  Mac OS X path
# This Python script loads/works with look-up table for brown carbon
import numpy
import struct

def GetDataLUT(coef1, coef2, coef3, radiusIndexMin, numRadiiGridBins, numScatteringAngles, Cext, Csca, C11, C12, C33, C34):
  indexMin = radiusIndexMin
  extCoef = Cext[0]*coef1[indexMin] + Cext[1]*coef2[indexMin] + Cext[2]*coef3[indexMin]
  scaCoef = Csca[0]*coef1[indexMin] + Csca[1]*coef2[indexMin] + Csca[2]*coef3[indexMin]
  p11 = C11[0]*coef1[indexMin] + C11[1]*coef2[indexMin] + C11[2]*coef3[indexMin]
  p12 = C12[0]*coef1[indexMin] + C12[1]*coef2[indexMin] + C12[2]*coef3[indexMin]
  p33 = C33[0]*coef1[indexMin] + C33[1]*coef2[indexMin] + C33[2]*coef3[indexMin]
  p34 = C34[0]*coef1[indexMin] + C34[1]*coef2[indexMin] + C34[2]*coef3[indexMin]
  indexMin += 1
  if (indexMin == 1):# Wavelength around 355 nm
    extCoef *= 2;  scaCoef *= 2
    p11 *= 2;  p12 *= 2;  p33 *= 2;  p34 *= 2;
    indexMin += 1
  numRad = numRadiiGridBins - indexMin

  extCoef += numpy.sum(Cext[0 : numRad]*coef1[indexMin : indexMin + numRad] + Cext[1 : 1 + numRad]*coef2[indexMin : indexMin + numRad] + Cext[2 : 2 + numRad]*coef3[indexMin : indexMin + numRad])
  scaCoef += numpy.sum(Csca[0 : numRad]*coef1[indexMin : indexMin + numRad] + Csca[1 : 1 + numRad]*coef2[indexMin : indexMin + numRad] + Csca[2 : 2 + numRad]*coef3[indexMin : indexMin + numRad])
  for m in range(0, numScatteringAngles):
    p11[m] += numpy.sum(C11[0 : numRad, m]*coef1[indexMin : indexMin + numRad] + C11[1 : 1 + numRad, m]*coef2[indexMin : indexMin + numRad] + C11[2 : 2 + numRad, m]*coef3[indexMin : indexMin + numRad])
    p12[m] += numpy.sum(C12[0 : numRad, m]*coef1[indexMin : indexMin + numRad] + C12[1 : 1 + numRad, m]*coef2[indexMin : indexMin + numRad] + C12[2 : 2 + numRad, m]*coef3[indexMin : indexMin + numRad])
    p33[m] += numpy.sum(C33[0 : numRad, m]*coef1[indexMin : indexMin + numRad] + C33[1 : 1 + numRad, m]*coef2[indexMin : indexMin + numRad] + C33[2 : 2 + numRad, m]*coef3[indexMin : indexMin + numRad])
    p34[m] += numpy.sum(C34[0 : numRad, m]*coef1[indexMin : indexMin + numRad] + C34[1 : 1 + numRad, m]*coef2[indexMin : indexMin + numRad] + C34[2 : 2 + numRad, m]*coef3[indexMin : indexMin + numRad])

  return extCoef, scaCoef, p11, p12, p33, p34



# Change the desired wavelength (mum)
wavelength = 0.55

# Bimodal   aerosol if numberOfModes = 2
# Monomodal aerosol if numberOfModes = 1
numberOfModes = 2

realPart = numpy.zeros(2)
imagPart = numpy.zeros(2)
countMedianRadius = numpy.zeros(2)
lnSigma = numpy.zeros(2)
numberConcentration = numpy.zeros(2)

# Complex refractive index
realPart[0] = 1.55
realPart[1] = 1.52
imagPart[0] = 0.04
imagPart[1] = 0.0019

# Lognormal PSD
countMedianRadius[0] = 0.12
countMedianRadius[1] = 0.5
lnSigma[0] = 0.37
lnSigma[1] = 0.75
numberConcentration[0] = 10
numberConcentration[1] = 5

with open("AerosolLUT_1000_100_0.355_650bins_2325CRI_ln2rKr_Twomey.dat", mode = 'rb') as file:
  print("Loading optimized Mie look-up table...")
  bufFloat               = struct.unpack('f', file.read(4))
  wavelengthLUT          = bufFloat[0]
  bufInt                 = struct.unpack('i', file.read(4))
  numRadiiGridBinsLUT    = bufInt[0]
  radiiGridBinsLUT       = numpy.zeros(numRadiiGridBinsLUT)
  radiiGridBinsLUT       = struct.unpack('f'*numRadiiGridBinsLUT, file.read(4*numRadiiGridBinsLUT))
  bufInt                 = struct.unpack('i', file.read(4))
  numScatteringAnglesLUT = bufInt[0]
  scatteringAnglesLUT    = numpy.zeros(numScatteringAnglesLUT)
  scatteringAnglesLUT    = struct.unpack('f'*numScatteringAnglesLUT, file.read(4*numScatteringAnglesLUT))
  bufInt                 = struct.unpack('i', file.read(4))
  numRealPartsLUT        = bufInt[0]
  realPartsLUT           = numpy.zeros(numRealPartsLUT)
  realPartsLUT           = struct.unpack('f'*numRealPartsLUT, file.read(4*numRealPartsLUT))
  bufInt                 = struct.unpack('i', file.read(4))
  numImagPartsLUT        = bufInt[0]
  imagPartsLUT           = numpy.zeros(numImagPartsLUT)
  imagPartsLUT           = struct.unpack('f'*numImagPartsLUT, file.read(4*numImagPartsLUT))
  numCRI = numRealPartsLUT*numImagPartsLUT
  realPartRecord = numpy.zeros(numCRI)
  imagPartRecord = numpy.zeros(numCRI)
  Cext = numpy.zeros(numCRI*numRadiiGridBinsLUT).reshape(numCRI, numRadiiGridBinsLUT)
  Csca = numpy.zeros(numCRI*numRadiiGridBinsLUT).reshape(numCRI, numRadiiGridBinsLUT)
  C11  = numpy.zeros(numCRI*numRadiiGridBinsLUT*numScatteringAnglesLUT).reshape(numCRI, numRadiiGridBinsLUT, numScatteringAnglesLUT)
  C12  = numpy.zeros(numCRI*numRadiiGridBinsLUT*numScatteringAnglesLUT).reshape(numCRI, numRadiiGridBinsLUT, numScatteringAnglesLUT)
  C33  = numpy.zeros(numCRI*numRadiiGridBinsLUT*numScatteringAnglesLUT).reshape(numCRI, numRadiiGridBinsLUT, numScatteringAnglesLUT)
  C34  = numpy.zeros(numCRI*numRadiiGridBinsLUT*numScatteringAnglesLUT).reshape(numCRI, numRadiiGridBinsLUT, numScatteringAnglesLUT)
  for i in range(0, numCRI):
    bufFloat = struct.unpack('f', file.read(4));  realPartRecord[i] = bufFloat[0]
    bufFloat = struct.unpack('f', file.read(4));  imagPartRecord[i] = bufFloat[0]
    Cext[i]  = struct.unpack('f'*numRadiiGridBinsLUT, file.read(4*numRadiiGridBinsLUT))
    Csca[i]  = struct.unpack('f'*numRadiiGridBinsLUT, file.read(4*numRadiiGridBinsLUT))
    for iGB in range(0, numRadiiGridBinsLUT):  C11[i][iGB] = struct.unpack('f'*numScatteringAnglesLUT, file.read(4*numScatteringAnglesLUT))
    for iGB in range(0, numRadiiGridBinsLUT):  C12[i][iGB] = struct.unpack('f'*numScatteringAnglesLUT, file.read(4*numScatteringAnglesLUT))
    for iGB in range(0, numRadiiGridBinsLUT):  C33[i][iGB] = struct.unpack('f'*numScatteringAnglesLUT, file.read(4*numScatteringAnglesLUT))
    for iGB in range(0, numRadiiGridBinsLUT):  C34[i][iGB] = struct.unpack('f'*numScatteringAnglesLUT, file.read(4*numScatteringAnglesLUT))
  file.close()

dV_dlnr = numpy.zeros(numRadiiGridBinsLUT)

extCoef = numpy.zeros(numberOfModes)
scaCoef = numpy.zeros(numberOfModes)
p11 = numpy.zeros(numberOfModes*numScatteringAnglesLUT).reshape(numberOfModes, numScatteringAnglesLUT)
p12 = numpy.zeros(numberOfModes*numScatteringAnglesLUT).reshape(numberOfModes, numScatteringAnglesLUT)
p33 = numpy.zeros(numberOfModes*numScatteringAnglesLUT).reshape(numberOfModes, numScatteringAnglesLUT)
p34 = numpy.zeros(numberOfModes*numScatteringAnglesLUT).reshape(numberOfModes, numScatteringAnglesLUT)

extCoefFinal = scaCoefFinal = 0
p11Final = numpy.zeros(numScatteringAnglesLUT)
p12Final = numpy.zeros(numScatteringAnglesLUT)
p33Final = numpy.zeros(numScatteringAnglesLUT)
p34Final = numpy.zeros(numScatteringAnglesLUT)
coefReal = numpy.zeros(3)
realPartIndex = numpy.zeros(3)
coefImag = numpy.zeros(3)
imagPartIndex = numpy.zeros(3)

wavelengthRatio = wavelengthLUT/wavelength
coef1 = numpy.zeros(numRadiiGridBinsLUT);
coef2 = numpy.zeros(numRadiiGridBinsLUT);
coef3 = numpy.zeros(numRadiiGridBinsLUT);
coef1PSD = numpy.zeros(numRadiiGridBinsLUT)
coef2PSD = numpy.zeros(numRadiiGridBinsLUT)
coef3PSD = numpy.zeros(numRadiiGridBinsLUT)
radiusIndex = [0] * numRadiiGridBinsLUT
bRadiusIndexNeedsInit = True
k = radiusIndexMin = 0
for iGB in range(0, numRadiiGridBinsLUT):
  radius = wavelengthRatio*radiiGridBinsLUT[iGB]
  radiusIndex[iGB] = -1
  for l in range(k, numRadiiGridBinsLUT - 1):
    if (radius >= radiiGridBinsLUT[l] and radius <= radiiGridBinsLUT[l + 1]):
      k = radiusIndex[iGB] = l
      if (bRadiusIndexNeedsInit): radiusIndexMin = iGB; bRadiusIndexNeedsInit = False
      if (l):	x1 = radiiGridBinsLUT[l - 1];	x2 = radiiGridBinsLUT[l];	x3 = radiiGridBinsLUT[l + 1]
      else:	x1 = radiiGridBinsLUT[l];	x2 = radiiGridBinsLUT[l + 1];	x3 = radiiGridBinsLUT[l + 2]
      coef1[iGB] = (radius - x2)*(radius - x3)/(x1 - x2)/(x1 - x3)
      coef2[iGB] = (radius - x1)*(radius - x3)/(x2 - x1)/(x2 - x3)
      coef3[iGB] = (radius - x1)*(radius - x2)/(x3 - x1)/(x3 - x2)
      break

final4 = gFinal = nFinal = reffFinal = sFinal = veffFinal = vFinal = 0
for iM in range(0, numberOfModes):
  dV_dlnr = numberConcentration[iM]*4./3.*numpy.pi/numpy.sqrt(2.*numpy.pi)*numpy.power(radiiGridBinsLUT, 3)/lnSigma[iM]*numpy.exp(-numpy.power(numpy.log(numpy.divide(radiiGridBinsLUT, countMedianRadius[iM])), 2)*0.5/lnSigma[iM]/lnSigma[iM])

  for iR in range(0, numRealPartsLUT - 1):
    if (realPart[iM] >= realPartsLUT[iR] and realPart[iM] <= realPartsLUT[iR + 1]): break
  if (realPart[iM] < realPartsLUT[0]):                   iR = 0
  if (realPart[iM] > realPartsLUT[numRealPartsLUT - 1]): iR = numRealPartsLUT - 2
  if (iR):
    x1 = realPartsLUT[iR - 1];  realPartIndex[0] = iR - 1
    x2 = realPartsLUT[iR];      realPartIndex[1] = iR
    x3 = realPartsLUT[iR + 1];  realPartIndex[2] = iR + 1
  else:
    x1 = realPartsLUT[iR];      realPartIndex[0] = iR
    x2 = realPartsLUT[iR + 1];  realPartIndex[1] = iR + 1
    x3 = realPartsLUT[iR + 2];  realPartIndex[2] = iR + 2
  coefReal[0] = (realPart[iM] - x2)*(realPart[iM] - x3)/(x1 - x2)/(x1 - x3)
  coefReal[1] = (realPart[iM] - x1)*(realPart[iM] - x3)/(x2 - x1)/(x2 - x3)
  coefReal[2] = (realPart[iM] - x1)*(realPart[iM] - x2)/(x3 - x1)/(x3 - x2)

  for iI in range(0, numImagPartsLUT - 1):
    if (imagPart[iM] >= imagPartsLUT[iI] and imagPart[iM] <= imagPartsLUT[iI + 1]): break
  if (imagPart[iM] < imagPartsLUT[0]):                   iI = 0
  if (imagPart[iM] > imagPartsLUT[numImagPartsLUT - 1]): iI = numImagPartsLUT - 2
  if (iI):
    x1 = imagPartsLUT[iI - 1];  imagPartIndex[0] = iI - 1
    x2 = imagPartsLUT[iI];      imagPartIndex[1] = iI
    x3 = imagPartsLUT[iI + 1];  imagPartIndex[2] = iI + 1
  else:
    x1 = imagPartsLUT[iI];      imagPartIndex[0] = iI
    x2 = imagPartsLUT[iI + 1];  imagPartIndex[1] = iI + 1
    x3 = imagPartsLUT[iI + 2];  imagPartIndex[2] = iI + 2
  coefImag[0] = (imagPart[iM] - x2)*(imagPart[iM] - x3)/(x1 - x2)/(x1 - x3)
  coefImag[1] = (imagPart[iM] - x1)*(imagPart[iM] - x3)/(x2 - x1)/(x2 - x3)
  coefImag[2] = (imagPart[iM] - x1)*(imagPart[iM] - x2)/(x3 - x1)/(x3 - x2)

  extCoef[iM] = scaCoef[iM] = 0
  p11[iM][:] = p12[iM][:] = p33[iM][:] = p34[iM][:] = 0
  coef1PSD = wavelengthRatio*coef1*dV_dlnr
  coef2PSD = wavelengthRatio*coef2*dV_dlnr
  coef3PSD = wavelengthRatio*coef3*dV_dlnr
  for j in range(0, 3):
    for k in range(0, 3):
      recordIndex = int(numImagPartsLUT*realPartIndex[j] + imagPartIndex[k])
      bufExt, bufSca, buf11, buf12, buf33, buf34 = GetDataLUT(coef1PSD, coef2PSD, coef3PSD, radiusIndexMin,
                                                              numRadiiGridBinsLUT, numScatteringAnglesLUT,
                                                              Cext[recordIndex], Csca[recordIndex], C11[recordIndex], C12[recordIndex], C33[recordIndex], C34[recordIndex])
      coefRealImag = coefReal[j]*coefImag[k]
      extCoef[iM] += bufExt*coefRealImag
      scaCoef[iM] += bufSca*coefRealImag
      p11[iM] += buf11*coefRealImag
      p12[iM] += buf12*coefRealImag
      p33[iM] += buf33*coefRealImag
      p34[iM] += buf34*coefRealImag

  extCoefFinal += extCoef[iM]
  scaCoefFinal += scaCoef[iM]
  p11Final += p11[iM]
  p12Final += p12[iM]
  p33Final += p33[iM]
  p34Final += p34[iM]
  gFinal += numberConcentration[iM]*numpy.pi*countMedianRadius[iM]*countMedianRadius[iM]*numpy.exp(2*lnSigma[iM]*lnSigma[iM]);
  nFinal += numberConcentration[iM];
  sFinal += numberConcentration[iM]*countMedianRadius[iM]*countMedianRadius[iM]*numpy.exp(2*lnSigma[iM]*lnSigma[iM]);
  vFinal += numberConcentration[iM]*countMedianRadius[iM]*countMedianRadius[iM]*countMedianRadius[iM]*numpy.exp(4.5*lnSigma[iM]*lnSigma[iM]);
  final4 += numberConcentration[iM]*countMedianRadius[iM]*countMedianRadius[iM]*countMedianRadius[iM]*countMedianRadius[iM]*numpy.exp(8*lnSigma[iM]*lnSigma[iM]);

absCoefFinal = numpy.abs(extCoefFinal - scaCoefFinal)
ssaFinal = scaCoefFinal/extCoefFinal
bscCoefFinal = p11Final[numScatteringAnglesLUT - 1]*0.25/numpy.pi
p11Final /= scaCoefFinal;  p12Final /= scaCoefFinal;  p33Final /= scaCoefFinal;  p34Final /= scaCoefFinal
p12Final /= p11Final;      p33Final /= p11Final;      p34Final /= p11Final
if (nFinal > 0):                                          gFinal /= nFinal;
if (numpy.abs(sFinal) > 0):                               reffFinal = vFinal/sFinal;
if (numpy.abs(sFinal) > 0 and numpy.abs(reffFinal) > 0):  veffFinal = 1 + (final4 - 2*reffFinal*vFinal)/sFinal/reffFinal/reffFinal;

asymmetryFinal = 0
numIntegrationAngles = 10001
for j in range(0, numIntegrationAngles):
  currentAngle = j*180./(numIntegrationAngles - 1)
  buf = 0
  for m in range(0, numScatteringAnglesLUT - 1):
    if ((currentAngle >= scatteringAnglesLUT[m]) and (currentAngle <= scatteringAnglesLUT[m + 1])):
      if (m < 0.5*numScatteringAnglesLUT):
        x1 = scatteringAnglesLUT[m];      y1 = numpy.log(p11Final[m])
        x2 = scatteringAnglesLUT[m + 1];  y2 = numpy.log(p11Final[m + 1])
        x3 = scatteringAnglesLUT[m + 2];  y3 = numpy.log(p11Final[m + 2])
      else:
        x1 = scatteringAnglesLUT[m - 1];  y1 = numpy.log(p11Final[m - 1])
        x2 = scatteringAnglesLUT[m];      y2 = numpy.log(p11Final[m])
        x3 = scatteringAnglesLUT[m + 1];  y3 = numpy.log(p11Final[m + 1])
      buf = y1*(currentAngle - x2)*(currentAngle - x3)/(x1 - x2)/(x1 - x3) + y2*(currentAngle - x1)*(currentAngle - x3)/(x2 - x1)/(x2 - x3) + y3*(currentAngle - x1)*(currentAngle - x2)/(x3 - x1)/(x3 - x2)
      buf = numpy.exp(buf)*numpy.sin(currentAngle*numpy.pi/180.)*numpy.cos(currentAngle*numpy.pi/180.)
      break
  # Numerical integration using Simpson's rule
  if ((j == 0) or (j == numIntegrationAngles - 1)):  asymmetryFinal += 0.5*buf
  else:
    if (j%2):  asymmetryFinal += 2*buf
    else:      asymmetryFinal += buf
asymmetryFinal *= numpy.pi/(numIntegrationAngles - 1)/3.

charBuf = "Output_%14.12f.txt" % wavelength
with open(charBuf, mode = 'w') as file:
  file.write("Wavelength = %14.12f mum\n\n" % wavelength)
  for iM in range(0, numberOfModes):
    file.write("Aerosol mode %d\n" % int(iM + 1))
    file.write("  Complex refractive index:\n")
    file.write("    real part = %14.12f\n" % realPart[iM])
    file.write("    imaginary part = %14.12f\n" % imagPart[iM])
    file.write("  Logarithmic-normal particle size distribution:\n")
    file.write("    total number concentration =%19.12e 1/cm^3\n" % numberConcentration[iM])
    file.write("    count median radius = %14.12f mum\n" % countMedianRadius[iM])
    file.write("    geometric standard deviation = %14.12f\n" % numpy.exp(lnSigma[iM]))
    file.write("    <g> =%19.12e mum^2\n" % numpy.float64(numpy.pi*countMedianRadius[iM]*countMedianRadius[iM]*numpy.exp(2*lnSigma[iM]*lnSigma[iM])))
    file.write("    effective radius =%19.12e mum\n" % numpy.float64(countMedianRadius[iM]*numpy.exp(2.5*lnSigma[iM]*lnSigma[iM])))
    file.write("    effective variance =%19.12e\n" % numpy.float64(numpy.exp(lnSigma[iM]*lnSigma[iM]) - 1))
  file.write("\nOptical coefficients:\n")
  file.write("  absorption     =%19.12e 1/Mm\n" % absCoefFinal)
  file.write("  extinction     =%19.12e 1/Mm\n" % extCoefFinal)
  file.write("  scattering     =%19.12e 1/Mm\n" % scaCoefFinal)
  file.write("  backscattering =%19.12e 1/(Mm sr)\n" % bscCoefFinal)
  file.write("Single-scattering albedo = %14.12f\n" % ssaFinal)
  file.write("Asymmetry parameter =%19.12e\n" % asymmetryFinal)
  file.write("<G> =%19.12e mum^2\n" % gFinal)
  file.write("Effective radius =%19.12e mum\n" % reffFinal)
  file.write("Effective variance =%19.12e\n\n" % veffFinal)
  file.write("            ******* Normalized scattering matrix *******\n")
  file.write("Angle  P11           P12/P11       P33/P11       P34/P11\n")
  for m in range(0, numScatteringAnglesLUT):
    file.write("%5.1f %13.6e %13.6e %13.6e %13.6e\n" % (scatteringAnglesLUT[m], p11Final[m], p12Final[m], p33Final[m], p34Final[m]))
file.close()